searchData={"items":[{"type":"module","title":"Routex.Attrs","doc":"Provides an interface to access and update Routex attributes.\n\nExtensions can make use of `Routex.Attrs` values provided by Routex itself,\nRoutex backends and other extensions. As those values are attributes to a\nroute, extension B can use values attributed to a route by extension A.\n\n> #### In combination with... {: .neutral}\n> Other extensions set `Routex.Attrs`. The attributes an extension sets is listed in it's documentation.\n> To define custom attributes for routes have a look at `Routex.Extension.Alternatives`\n\n* To make the availability of the attributes as predictable as possible, Routex\nuses a flat structure.\n* Extension developers are encouraged to put as much information into the attributes\nas possible.\n* Extensions should add any fallback/default they might use themselves to the\nattributes.","ref":"Routex.Attrs.html"},{"type":"function","title":"Routex.Attrs.cleanup/1","doc":"Removes non private fields from attrs. Input is either an opts map or a `Route`.","ref":"Routex.Attrs.html#cleanup/1"},{"type":"function","title":"Routex.Attrs.get/3","doc":"Get the value assigned to `key` from `route`'s options or return `default`. When no\nkey is provided, the function returns all options.","ref":"Routex.Attrs.html#get/3"},{"type":"function","title":"Routex.Attrs.get!/3","doc":"Get the value assigned to `key` from `route`'s options or raise if `key` not found. Accepts an optional\ncustom `error_msg` as third argument","ref":"Routex.Attrs.html#get!/3"},{"type":"function","title":"Routex.Attrs.merge/2","doc":"Merge `value` into `route`'s options.","ref":"Routex.Attrs.html#merge/2"},{"type":"function","title":"Routex.Attrs.private?/1","doc":"Returns true when the provided key or attribute is private.","ref":"Routex.Attrs.html#private?/1"},{"type":"function","title":"Routex.Attrs.put/2","doc":"Assigns `value` to `route`'s options.","ref":"Routex.Attrs.html#put/2"},{"type":"function","title":"Routex.Attrs.put/3","doc":"Assigns `value` to `key` in `route`'s options.","ref":"Routex.Attrs.html#put/3"},{"type":"function","title":"Routex.Attrs.update/2","doc":"Updates the options of a `route` by applying given `function`.","ref":"Routex.Attrs.html#update/2"},{"type":"function","title":"Routex.Attrs.update/3","doc":"Updates the value assigned to `key` of a `route`'s options by applying given `function`.","ref":"Routex.Attrs.html#update/3"},{"type":"module","title":"Routex.Backend","doc":"> #### `use Routex.Backend` {: .info}\n> When use'd this module generates a Routex backend module and\n> a configuration struct using the `configure/2` callbacks of\n> the extensions provided in `opts`.\n>\n> See also: [Routex Extensions](EXTENSION_DEVELOPMENT.md).\n\n**Example**\n\n     iex> defmodule MyApp.RtxBackend do\n     ...>  use Routex.Backend,\n     ...>   extensions: [\n     ...>    Routex.Extension.VerifiedRoutes,\n     ...>    Routex.Extension.AttrGetters,\n     ...>   ],\n     ...>   extension_x_config: [key: \"value\"]\n     ...> end\n     iex> IO.inspect(%MyApp.RtxBackend{})\n     %MyApp.RtxBackend{\n       extension_x_config: [key: \"value\"],\n       extensions: [Routex.Extension.VerifiedRoutes, Routex.Extension.AttrGetters],\n       verified_sigil_routex: \"~l\",\n       verified_sigil_original: \"~o\"\n     }\n\nValues in the configuration can be overridden by providing an override map to the `:private` option of a scope or route.\n\n**Example**\n\n    live /products, MyApp.Web.ProductIndexLive, :index, private: %{rtx: %{overridden_key: value}}","ref":"Routex.Backend.html"},{"type":"type","title":"Routex.Backend.t/0","doc":"A Routex backend module","ref":"Routex.Backend.html#t:t/0"},{"type":"module","title":"Routex.Branching","doc":"Provides a function to build branched variants of macro's","ref":"Routex.Branching.html"},{"type":"function","title":"Routex.Branching.branch_macro/5","doc":"Takes a list of match patterns and creates the AST for branching variants for\nall arities of `macro` in `module` by wrapping them in a `case` statement.\n\n\n** Args **\n\n- `patterns`: the match patterns to be used as case clauses\n- `match_binding`: ast inserted as `case [match_binding] do`\n- `module`: the module name\n- `macro`: the macro name\n- `opts`: list of options\n\n\n** Options **\n- `arities`: a list of arities to transform, default: all arities\n- `as`: name of the branching variant. default: the macro name\n- `arg_post`: function to calculate the replaced argument position. default: 0\n\nThe clauses and arguments can be transformed by providing MFA's. The\ntransformers receive as arguments the `pattern`, the `banched arg` and any\nother argument provided in `a`.\n\n- `clause_transformer`: {m,f,a}. transforms a pattern; used as case clause in the macro body.\n- `arg_transformer`: {m,f,a}.  transforms a branched argument; used in the macro body.\n\n** Example **\n\nWe want to create a branching variant of the `url` macro in `Phoenix.VerifiedRoutes` module. The original\nmacro generates code that simply prints the given path argument, but we want to it to write multiple clauses and\nprefix the given argument based on the clause.\n\n    defmacro url(path, opts \\ []) do -> quote do IO.puts(path) end\n\nGiven this code:\n\n    defmodule MyMod do\n      def transform_arg(pattern, arg, extra), do: \"/\" <> extra <> \"/europe/\" <> pattern <> \"/\" <> arg end\n    end\n\n    patterns = [\"en\", \"nl\"]\n    match_binding = var!(external_var)\n    arg_pos = fn arity -> arity - 1 end)\n    arg_transformer = {MyMod, transform_arg, [\"my_extra\"]}\n    opts = [as: :url, orig: :url_original, arg_pos: arg_pos, arg_transformer: arg_transformer]\n\n    branch_macro(patterns, match_binding, OriginalModule, :url, opts)\n\nA new macro is build which outputs the AST of the original macro, wrapped in a case clause given transformed arguments.\n\n    defmacro url(path, opts \\ []) do\n\t    quote do\n\t      case external_var do\n\t  \t\t \"en\" -> Original.Module.url( \"/\" <> \"my_extra\" <> \"/europe/en/\" <> path, opts)\n\t  \t\t \"nl\" -> Original.Module.url(\"/\" <> \"my_extra\" <> \"/europe/nl/\" <> path, opts)\n\t  \t end\n       end\n     end\n\n\nFor more examples, please see the test module `Routex.BranchingTest`.","ref":"Routex.Branching.html#branch_macro/5"},{"type":"module","title":"Routex.Dev","doc":"Provides functions to aid during development","ref":"Routex.Dev.html"},{"type":"function","title":"Routex.Dev.inspect_ast/2","doc":"`Macro.escape/1` and `IO.inspect/2` the given input. Options are\npassed through to `IO.inspect`. Returns the input.","ref":"Routex.Dev.html#inspect_ast/2"},{"type":"function","title":"Routex.Dev.print_ast/2","doc":"Helper function to inspect AST as formatted code. Returns the\ninput.","ref":"Routex.Dev.html#print_ast/2"},{"type":"function","title":"Examples - Routex.Dev.print_ast/2","doc":"iex> ast = quote do: Map.put(my_map, :key, value)\n    iex> print_ast(ast)\n    Map.put(my_map, :key, value)\n    ...actual AST...","ref":"Routex.Dev.html#print_ast/2-examples"},{"type":"behaviour","title":"Routex.Extension","doc":"Specification for composable Routex extensions.\n\nOptional callbacks:\n- configure\n- transform\n- post_transform\n- create_helpers\n\nSee also: [Routex Extensions](EXTENSIONS.md)","ref":"Routex.Extension.html"},{"type":"callback","title":"Routex.Extension.configure/2","doc":"The `configure/2` callback is called in the first stage with the options\nprovided to `Routex` and the name of the Routex backend. It is expected to\nreturn a new list of options.","ref":"Routex.Extension.html#c:configure/2"},{"type":"callback","title":"Routex.Extension.create_helpers/3","doc":"The `create_helpers/3` callback is called in the last stage with a list of\nroutes belonging to a Routex backend, the name of the Routex backend and\nthe current environment. It is expected to return Elixir AST.\n\nThe AST is included in `MyAppWeb.Router.RoutexHelpers`.","ref":"Routex.Extension.html#c:create_helpers/3"},{"type":"callback","title":"Routex.Extension.post_transform/3","doc":"The `post_transform/1` callback is called in the third stage with a list of\nroutes belonging to a Routex backend. It is expected to return a list of\nPhoenix.Router.Route structs almost identical to the input, only adding\n`Routex.Attrs` -for own usage- is allowed.","ref":"Routex.Extension.html#c:post_transform/3"},{"type":"callback","title":"Routex.Extension.transform/3","doc":"The `transform/3` callback is called in the second stage with a list of\nroutes belonging to a Routex backend, the name of the configuration model\nand the current environment. It is expected to return a list of\nPhoenix.Router.Route structs with flattened `Routex.Attrs`.","ref":"Routex.Extension.html#c:transform/3"},{"type":"type","title":"Routex.Extension.backend/0","doc":"","ref":"Routex.Extension.html#t:backend/0"},{"type":"type","title":"Routex.Extension.env/0","doc":"","ref":"Routex.Extension.html#t:env/0"},{"type":"type","title":"Routex.Extension.opts/0","doc":"","ref":"Routex.Extension.html#t:opts/0"},{"type":"type","title":"Routex.Extension.routes/0","doc":"","ref":"Routex.Extension.html#t:routes/0"},{"type":"module","title":"Routex.Matchable","doc":"Matchables are an essential part of Routex. They are used to match run time\nroutes with compile time routes and enable reordered route segments.\n\nThis module provides functions to create Matchables, convert them to match\npattern AST as well as function head AST, and check if the routing values\nof two Matchable records match.","ref":"Routex.Matchable.html"},{"type":"function","title":"Routex.Matchable.match?/2","doc":"Returns whether two Matchable records match on their route defining\nproperties. The first argument supports param en wildcard syntax\n(e.g \":param\" and \"*\").","ref":"Routex.Matchable.html#match?/2"},{"type":"function","title":"Example - Routex.Matchable.match?/2","doc":"iex> route_record = %Phoenix.Router.Route{path: \"/posts/:id\"} |> Routex.Matchable.new()\n    iex> matching_record = \"/posts/1/foo=bar#top\" |> Routex.Matchable.new()\n    iex> non_matching_record = \"/other/1/foo=bar#op\" |> Routex.Matchable.new()\n\n    iex> match?(route_record, matching_record)\n    true\n\n    iex match?(route_record, non_matching_record)\n    false","ref":"Routex.Matchable.html#match?/2-example"},{"type":"macro","title":"Routex.Matchable.matchable/1","doc":"","ref":"Routex.Matchable.html#matchable/1"},{"type":"macro","title":"Routex.Matchable.matchable/2","doc":"","ref":"Routex.Matchable.html#matchable/2"},{"type":"function","title":"Routex.Matchable.new/1","doc":"Converts a binary URL, `Phoenix.Router.Route` or (sigil) AST argument into a Matchable record.","ref":"Routex.Matchable.html#new/1"},{"type":"function","title":"Examples - Routex.Matchable.new/1","doc":"iex> path = \"/posts/1?foo=bar#top\"\n  iex> route = %Phoenix.Router.Route{path: \"/posts/:id\"}\n  iex> ast = {:<<>>, [], [\"/products/\", {:\"::\", [], [{{:., [], [Kernel, :to_string]}, [from_interpolation: true], [{:id, [], Elixir}]}, {:binary, [], Elixir}]}]}\n\n\tiex> path_match = Routex.Matchable.new(path)\n\t{:matchable, [nil], [\"posts\", \"1\"], \"foo=bar\", \"top\", false}\n\n\tiex> route_match = Routex.Matchable.new(route)\n\t{:matchable, [], [\"posts\", \":id\"], nil, nil, false}\n\n\tiex> ast_match = Routex.Matchable.new(ast)\n\t{:matchable, [], [\"posts\", {:\"::\", [], [{{:., [], [Kernel, :to_string]}, [from_interpolation: true], [{:id, [], Elixir}]}, {:binary, [], Elixir}]}], nil, nil, false}","ref":"Routex.Matchable.html#new/1-examples"},{"type":"function","title":"Routex.Matchable.placeholders_to_ast/2","doc":"","ref":"Routex.Matchable.html#placeholders_to_ast/2"},{"type":"function","title":"Routex.Matchable.to_ast_segments/1","doc":"","ref":"Routex.Matchable.html#to_ast_segments/1"},{"type":"function","title":"Routex.Matchable.to_func/4","doc":"Creates a function named `name` which the first argument matching\na Matchable record pattern. Other arguments can be given with either a\ncatch all or a pattern.\n\nThe Matchable pattern is bound to variable `pattern`","ref":"Routex.Matchable.html#to_func/4"},{"type":"function","title":"Example - Routex.Matchable.to_func/4","doc":"iex> \"/some/path\"\n       >  |> Matchable.new()\n       >  |> Matchable.to_func(:my_func, [pattern_arg: \"fixed\", :catchall_arg], quote(do: :ok))","ref":"Routex.Matchable.html#to_func/4-example"},{"type":"function","title":"Routex.Matchable.to_pattern/1","doc":"Returns a match pattern for given `Matchable` record or `Phoenix.Router.Route`.\nThe pattern can be used either as function argument or in a function body. As\nthe pattern binds values, the bindings can be used to convert input from one\npattern to another.","ref":"Routex.Matchable.html#to_pattern/1"},{"type":"function","title":"Examples - Routex.Matchable.to_pattern/1","doc":"iex> \"/original/:arg1/:arg2\" |> Routex.Matchable.new() |> Routex.Matchable.to_pattern()\n\t{:{}, [], [:matchable, {:hosts, [], Routex.Matchable}, [\"original\", {:arg1, [], Routex.Matchable}, {:arg2, [], Routex.Matchable}], {:query, [], Routex.Matchable}, {:fragment, [], Routex.Matchable}, false]}\n\n\tiex> \"/recomposed/:arg2/:arg1\" |> Routex.Matchable.new() |> Routex.Matchable.to_pattern()\n\t{:{}, [], [:matchable, {:hosts, [], Routex.Matchable}, [\"recomposed\", {:arg2, [], Routex.Matchable}, {:arg1, [], Routex.Matchable}], {:query, [], Routex.Matchable}, {:fragment, [], Routex.Matchable}, false]}\n\n\n\tiex> \"/original/segment_1/segment_2\" |> Routex.Matchable.new() |> Routex.Matchable.to_pattern()\n\t{:{}, [], [:matchable, {:hosts, [], Routex.Matchable}, [\"original\", \"segment_1\", \"segment_2\"], {:query, [], Routex.Matchable}, {:fragment, [], Routex.Matchable}, false]}","ref":"Routex.Matchable.html#to_pattern/1-examples"},{"type":"function","title":"Routex.Matchable.to_string/1","doc":"A non conflicting function mimicking `to_string/1`","ref":"Routex.Matchable.html#to_string/1"},{"type":"module","title":"Routex.Processing","doc":"This module provides everything needed to process Phoenix routes. It executes\nthe `transform` callbacks from extensions to transform `Phoenix.Router.Route`\nstructs and `create_helpers` callbacks to create one unified Helper module.\n\n**Powerful but thin**\nAlthough Routex is able to influence the routes in Phoenix applications in profound\nways, the framework and it's extensions are a suprisingly lightweight piece\nof compile-time middleware. This is made possible by the way router modules\nare pre-processed by `Phoenix.Router` itself.\n\nPrior to compilation of a router module, Phoenix Router registers all routes\ndefined in the router module using the attribute `@phoenix_routes`. Each\nroute is at that stage a `Phoenix.Router.Route` struct.\n\nAny route enclosed in a `preprocess_using` block has received a `:private`\nfield in which Routex has put which Routex backend to use for that\nparticular route. By enumerating the routes, we can process each route using\nthe properties of this configuration and set struct values accordingly. This\nprocessing is nothing more than (re)mapping the Route structs' values.\n\nAfter the processing by Routex is finished, the `@phoenix_routes` attribute\nin the router is erased and re-populated with the list of mapped\nPhoenix.Router.Route structs.\n\nOnce the router module enters the compilation stage, Routex is already out of\nthe picture and route code generation is performed by Phoenix Router.","ref":"Routex.Processing.html"},{"type":"function","title":"Routex.Processing.__before_compile__/1","doc":"Callback executed before compilation of a `Phoenix Router`. This callback is added\nto the `@before_compile` callbacks by `Routex.Router`.","ref":"Routex.Processing.html#__before_compile__/1"},{"type":"function","title":"Routex.Processing.exec_when_defined/5","doc":"Checks if the `callback` is defined. When defined it executes\nthe `callback` and returns the result , otherwise returns `default`.","ref":"Routex.Processing.html#exec_when_defined/5"},{"type":"function","title":"Routex.Processing.execute_callbacks/1","doc":"The main function of this module. Receives as only argument the environment of a\nPhoenix router module.","ref":"Routex.Processing.html#execute_callbacks/1"},{"type":"type","title":"Routex.Processing.backend/0","doc":"","ref":"Routex.Processing.html#t:backend/0"},{"type":"type","title":"Routex.Processing.extension_module/0","doc":"","ref":"Routex.Processing.html#t:extension_module/0"},{"type":"type","title":"Routex.Processing.routes/0","doc":"","ref":"Routex.Processing.html#t:routes/0"},{"type":"module","title":"Routex.Route","doc":"Function for working with Routex augmented Phoenix Routes","ref":"Routex.Route.html"},{"type":"function","title":"Routex.Route.exprs/2","doc":"Compatibility wrapper around `Phoenix.Router.Route.exprs`","ref":"Routex.Route.html#exprs/2"},{"type":"function","title":"Routex.Route.get_nesting/2","doc":"Returns the nesting level of an (ancestor) route. By default\nthe parent. This can be adjusted by providing an negative depth offset.","ref":"Routex.Route.html#get_nesting/2"},{"type":"function","title":"Routex.Route.group_by_method_and_origin/1","doc":"Returns routes grouped by the combination of method and origin path","ref":"Routex.Route.html#group_by_method_and_origin/1"},{"type":"function","title":"Routex.Route.group_by_method_and_path/2","doc":"","ref":"Routex.Route.html#group_by_method_and_path/2"},{"type":"function","title":"Routex.Route.group_by_nesting/2","doc":"Returns routes grouped by nesting level of an (ancestor) route. By default\ngroups by parent. This can be adjusted by providing an negative depth offset","ref":"Routex.Route.html#group_by_nesting/2"},{"type":"module","title":"Routex.Router","doc":"Provides macro (callbacks) to alter route definition before\ncompilation.\n\n> #### `use Routex.Router` {: .info}\n>\n> When you `use Routex.Router`, the Routex.Router module will\n> plug `Routex.Processing` between the definition of routes and the\n> compilation of the router module. It also imports the `preprocess_using`\n> macro which can be used to mark routes for Routex preprocessing using the\n> Routex backend provided as first argument.","ref":"Routex.Router.html"},{"type":"macro","title":"Routex.Router.preprocess_using/3","doc":"Wraps each enclosed route in a scope, marking it for processing by Routex\nusing given `backend`. `opts` can be used to partially override the given\nconfiguration.\n\nReplaces interpolation syntax with a string for macro free processing by\nextensions. Format: `[rtx.{binding}]`.","ref":"Routex.Router.html#preprocess_using/3"},{"type":"module","title":"Routex.Utils","doc":"Provides an interface to functions which can be used in extensions.","ref":"Routex.Utils.html"},{"type":"function","title":"Routex.Utils.alert/1","doc":"Prints an alert. Should be used when printing critical alerts in\nthe terminal during compile time.","ref":"Routex.Utils.html#alert/1"},{"type":"function","title":"Routex.Utils.get_branch/1","doc":"","ref":"Routex.Utils.html#get_branch/1"},{"type":"function","title":"Routex.Utils.get_helper_ast/1","doc":"Returns the AST to get the current branch from process dict or from  assigns, conn or socket\nbased on the available variables in the `caller` module.","ref":"Routex.Utils.html#get_helper_ast/1"},{"type":"function","title":"Routex.Utils.print/1","doc":"Prints an indented text. Should be used when printing messages in\nthe terminal during compile time.","ref":"Routex.Utils.html#print/1"},{"type":"module","title":"Routex.Extension.AlternativeGetters","doc":"Creates helper functions to get a list of maps alternative slugs and their `Routex.Attrs`\nby providing a binary url. Sets `match?: true` for the url matching record.","ref":"Routex.Extension.AlternativeGetters.html"},{"type":"module","title":"Configuration - Routex.Extension.AlternativeGetters","doc":"```diff\n# file /lib/example_web/routex_backend.ex\ndefmodule ExampleWeb.RoutexBackend do\n  use Routex.Backend,\n  extensions: [\n    Routex.Extension.AttrGetters, # required\n    Routex.Extension.Alternatives,\n+   Routex.Extension.AlternativeGetters\n],\n```","ref":"Routex.Extension.AlternativeGetters.html#module-configuration"},{"type":"module","title":"Usage example - Routex.Extension.AlternativeGetters","doc":"```elixir\n<!-- @url is made available by Routex -->\n<!-- alternatives/1 is located in ExampleWeb.Router.RoutexHelpers aliased as Routes -->\n<.link\n   :for={alternative <- Routes.alternatives(@url)}\n   class=\"button\"\n   rel=\"alternate\"\n   hreflang={alternative.attrs.locale}\n   patch={alternative.slug}\n >\n   <.button class={(alternative.match? && \"highlighted\") || \"\"}>\n     <%= alternative.attrs.display_name %>\n   </.button>\n </.link>\n```","ref":"Routex.Extension.AlternativeGetters.html#module-usage-example"},{"type":"module","title":"Pseudo result - Routex.Extension.AlternativeGetters","doc":"```elixir\niex> ExampleWeb.Router.RoutexHelpers.alternatives(\"/products/12?foo=baz\")\n[ %Routex.Extension.AlternativeGetters{\n  slug: \"products/12/?foo=baz\",\n  match?: true,\n  attrs: %{\n    __branch__: [0, 12, 0],\n    __origin__: \"/products/:id\",\n    [...attributes set by other extensions...]\n  }},\n  %Routex.Extension.AlternativeGetters{\n  slug: \"/europe/products/12/?foo=baz\",\n  match?: false,\n  attrs: %{\n    __branch__: [0, 12, 1],\n    __origin__: \"/products/:id\",\n    [...attributes set by other extensions...]\n  }},\n %Routex.Extension.AlternativeGetters{\n  slug: \"/asia/products/12/?foo=baz\",\n  match?: false,\n  attrs: %{\n    __branch__: [0, 12, 1],\n    __origin__: \"/products/:id\",\n    [...attributes set by other extensions...]\n  }},\n]\n```\n\n## `Routex.Attrs`\n**Requires**\n- none\n\n**Sets**\n- none","ref":"Routex.Extension.AlternativeGetters.html#module-pseudo-result"},{"type":"module","title":"Helpers - Routex.Extension.AlternativeGetters","doc":"- alternatives(url :: String.t()) :: struct()","ref":"Routex.Extension.AlternativeGetters.html#module-helpers"},{"type":"module","title":"Routex.Extension.Alternatives","doc":"Creates alternative routes based on `branches` configured in a Routex backend\nmodule. Branches can be nested and each branch can provide `Routex.Attrs` to be shared\nwith other extensions.\n\n> #### In combination with... {: .neutral}\n> How to combine this extension for localization is written in de [Localization Guide](guides/LOCALIZED_ROUTES.md)","ref":"Routex.Extension.Alternatives.html"},{"type":"module","title":"Configuration - Routex.Extension.Alternatives","doc":"```diff\n# file /lib/example_web/routex_backend.ex\n# This example uses a `Struct` for custom attributes, so there is no attribute inheritance;\n# only struct defaults. When using maps, nested branches will inherit attributes from their parent.\n\n+ defmodule ExampleWeb.RoutexBackend.AltAttrs do\n+  @moduledoc false\n+  defstruct [:contact, locale: \"en\"]\n+ end\n\ndefmodule ExampleWeb.RoutexBackend do\n+ alias ExampleWeb.RoutexBackend.AltAttrs\n\nuse Routex.Backend,\nextensions: [\n  Routex.Extension.AttrGetters, # required\n+ Routex.Extension.Alternatives,\nRoutex.Extension.AttrGetters\n],\n+ alternatives: %{\n+    \"/\" => %{\n+      attrs: %AltAttrs{contact: \"root@example.com\"},\n+      branches: %{\n+        \"/europe\" => %{\n+          attrs: %AltAttrs{contact: \"europe@example.com\"},\n+          branches: %{\n+            \"/nl\" => %{attrs: %AltAttrs{locale: \"nl\", contact: \"verkoop@example.nl\"}},\n+            \"/be\" => %{attrs: %AltAttrs{locale: \"nl\", contact: \"handel@example.be\"}}\n+          }\n+        },\n+      \"/gb\" => %{attrs: %AltAttrs{contact: \"sales@example.com\"}\n+    }\n+  },\n+ alternatives_prefix: false  # whether to automatically prefix routes, defaults to true\n```","ref":"Routex.Extension.Alternatives.html#module-configuration"},{"type":"module","title":"Pseudo result - Routex.Extension.Alternatives","doc":"```elixir\n    Router              Generated                         Attributes\n                        ⇒ /products/:id/edit              locale: \"en\", contact: \"rootexample.com\"\n    /products/:id/edit  ⇒ /europe/nl/products/:id/edit    locale: \"nl\", contact: \"verkoop@example.nl\"\n                        ⇒ /europe/be/products/:id/edit    locale: \"nl\", contact: \"handel@example.be\"\n                        ⇒ /gb/products/:id/edit           locale: \"en\", contact: \"sales@example.com\"\n ```\n\n## `Routex.Attrs`\n**Requires**\n- none\n\n**Sets**\n- **any key/value in `:attrs`**\n- branch_helper\n- branch_alias\n- branch_prefix\n- branch_opts\n- alternatives (list of `Phoenix.Route.Route`)","ref":"Routex.Extension.Alternatives.html#module-pseudo-result"},{"type":"module","title":"Routex.Extension.Assigns","doc":"Extracts `Routex.Attrs` from a route and makes them available in components\nand controllers with the assigns operator `@` (optionally under a namespace).\n\n> #### In combination with... {: .neutral}\n> Other extensions set `Routex.Attrs`. The attributes an extension sets is listed in it's documentation.\n> To define custom attributes for routes have a look at `Routex.Extension.Alternatives`","ref":"Routex.Extension.Assigns.html"},{"type":"module","title":"Options - Routex.Extension.Assigns","doc":"- `namespace`: when set creates a named collection of `Routex.Attrs`\n- `attrs`: when set defines keys of `Routex.Attrs` to make available","ref":"Routex.Extension.Assigns.html#module-options"},{"type":"module","title":"Configuration - Routex.Extension.Assigns","doc":"```diff\n# file /lib/example_web/routex_backend.ex\ndefmodule ExampleWeb.RoutexBackend do\n  use Routex.Backend,\n  extensions: [\n    Routex.Extension.AttrGetters, # required\n+   Routex.Extension.Assigns\n],\n+ assigns: %{namespace: :rtx, attrs: [:branch_helper, :locale, :contact, :name]}\n```","ref":"Routex.Extension.Assigns.html#module-configuration"},{"type":"module","title":"Pseudo result - Routex.Extension.Assigns","doc":"# in (h)eex template\n    @rtx.branch_helper  ⇒  \"eu_nl\"\n    @rtx.locale         ⇒  \"nl\"\n    @rtx.contact        ⇒  \"verkoop@example.nl\"\n    @rtx.name           ⇒  \"The Netherlands\"\n\n## `Routex.Attrs`\n**Requires**\n- none\n\n**Sets**\n- assigns","ref":"Routex.Extension.Assigns.html#module-pseudo-result"},{"type":"module","title":"Example use case - Routex.Extension.Assigns","doc":"Combine with `Routex.Extension.Alternatives` to make compile time, branch\nbound assigns available to components and controllers.","ref":"Routex.Extension.Assigns.html#module-example-use-case"},{"type":"module","title":"Routex.Extension.AttrGetters","doc":"Creates helper functions to get the `Routex.Attrs` given a binary url or a\nlist of path segments. Use this to lazy load attributes instead of adding them\nupfront to assigns.\n\nThis extension provides the required `attrs/1` helper function, used by\nRoutex to assign helper attributes in the generated `on_mount/4` callback.\n\n> #### In combination with... {: .neutral}\n> Other extensions set `Routex.Attrs`. The attributes an extension sets is listed in it's documentation.\n> To define custom attributes for routes have a look at `Routex.Extension.Alternatives`","ref":"Routex.Extension.AttrGetters.html"},{"type":"module","title":"Configuration - Routex.Extension.AttrGetters","doc":"```diff\n# file /lib/example_web/routex_backend.ex\ndefmodule ExampleWeb.RoutexBackend do\n  use Routex.Backend,\n  extensions: [\n+   Routex.Extension.AttrGetters,  # required\n],\n```","ref":"Routex.Extension.AttrGetters.html#module-configuration"},{"type":"module","title":"Pseudo result - Routex.Extension.AttrGetters","doc":"```elixir\niex> ExampleWeb.Router.RoutexHelpers.attrs(\"/europe/nl/producten/?foo=baz\")\n%{\n  __branch__: [0, 9, 3],\n  __origin__: \"/products\",\n  backend: ExampleWeb.LocalizedRoutes,\n  contact: \"verkoop@example.nl\",\n  locale: \"nl\",\n  branch_name: \"The Netherlands\",\n  branch_helper: \"europe_nl\",\n}\n```\n\n## `Routex.Attrs`\n**Requires**\n- none\n\n**Sets**\n- none","ref":"Routex.Extension.AttrGetters.html#module-pseudo-result"},{"type":"module","title":"Helpers - Routex.Extension.AttrGetters","doc":"- attrs(url :: binary) :: map()\n- attrs(segments :: list) :: map()","ref":"Routex.Extension.AttrGetters.html#module-helpers"},{"type":"module","title":"Routex.Extension.Cldr","doc":"Adapter for projects using :ex_cldr. It generates the configuration\nfor `Routex.Extension.Alternatives`.\n\n> #### Ejecting the CLDR extension {: .neutral}\n> Using the Cldr adapter provides the advantage of keeping your localized routes\n> in sync with the configuration of Cldr. The disadvantage is a lack of flexibility.\n> If you ever need more flexibility, you can [eject the Cldr extension](#module-eject-the-cldr-adapter).","ref":"Routex.Extension.Cldr.html"},{"type":"module","title":"Interpolating Locale Data - Routex.Extension.Cldr","doc":"Interpolation is provided by `Routex.Extension.Interpolation`, which\nis able to use any `Routex.Attr` for interpolation into your routes.\nSee it's documentation for additional options.\n\nWhen using this Cldr extension, the following interpolations are supported as they\nare set as `Routex.Attr`:\n\n* `locale` will interpolate the Cldr locale name\n* `locale_display` will interpolate the Cldr locale display name\n* `language` will interpolate the Cldr language name\n* `territory` will interpolate the Cldr territory code\n\nSome examples:\n```elixir\npreprocess_using ExampleWeb.RoutexBackend do\n  scope \"/#{territory}/territory/\" do\n    get \"/locale/pages/:page/#{locale}/\", PageController, :show\n    get \"/language/#{language}/pages/:page\", PageController, :show\n  end\nend\n```","ref":"Routex.Extension.Cldr.html#module-interpolating-locale-data"},{"type":"module","title":"Configuration - Routex.Extension.Cldr","doc":"```diff\ndefmodule ExampleWeb.RoutexBackend do\nuse Routex.Backend,\nextensions: [\n  Routex.Extension.AttrGetters, # required\n+ Routex.Extension.Cldr,\n+ Routex.Extension.Alternatives,\n+ Routex.Extension.Interpolation, #  when using routes with interpolation\n+ Routex.Extension.Translations,  # when using translated routes\n+ Routex.Extension.VerifiedRoutes\n],\n+ cldr_backend: MyApp.Cldr,\n+ translations_backend: MyApp.Gettext,  #  when using translated routes\n+ translations_domain: \"routes\",  #  when using translated routes\n+ alternatives_prefix: false,  #  when using routes with interpolation\n+ verified_sigil_routex: \"~q\", #  consider using ~p, see `Routex.Extension.VerifiedRoutes`\n```\n\n```diff\ndefmodule ExampleWeb.Router\n# require your Cldr backend module before `use`ing the router.\n+ require ExampleWeb.Cldr\n\nuse ExampleWeb, :router\n\nimport ExampleWeb.UserAuth\n```\n\nWhen your application does not compile after adding this extension, force a\nrecompile using `mix compile --force`.","ref":"Routex.Extension.Cldr.html#module-configuration"},{"type":"module","title":"Pseudo result - Routex.Extension.Cldr","doc":"This extension injects `:alternatives` into your configuration.\n See the documentation of `Routex.Extension.Alternatives` to see\n more options and the pseudo result.","ref":"Routex.Extension.Cldr.html#module-pseudo-result"},{"type":"module","title":"Eject the Cldr adapter - Routex.Extension.Cldr","doc":"This extension abstracts away the configuration of `Routex.Extension.Alternatives`. You may want\nto customize things beyond what `Routex.Extension.Cldr` provides. When you eject, you copy\nthe generated configuration into the Routex backend.\n\nIn other words, instead of relying on the preconfigured “black box” provided by this extension, you\nnow have full access to—and responsibility for—the configuration of `Routex.Extension.Alternatives`.\n\n#### Copy the generated configuration into your Routex backend**\n\nCall the `config/0` function on you backend (e.g. `ExampleWeb.RoutexBackend.config()`)\nin IEX. Copy the `alternatives: %{...}` section to your Routex backend.\n\n```diff\ndefmodule ExampleWeb.RoutexBackend do\nuse Routex.Backend,\nextensions: [...],\n+ alternatives: %{...}\n```\n\n#### Remove references to Cldr\n\n```diff\ndefmodule ExampleWeb.RoutexBackend do\nuse Routex.Backend,\nextensions: [\n-  Routex.Extension.Cldr,\n],\n- cldr_backend: MyApp.Cldr,\n```\n\n```diff\ndefmodule ExampleWeb.Router\n- require ExampleWeb.Cldr\n\nuse ExampleWeb, :router\n\nimport ExampleWeb.UserAuth\n```\n\n## `Routex.Attrs`\n**Requires**\n- none\n\n**Sets**\n- language\n- locale\n- locale_display\n- territory","ref":"Routex.Extension.Cldr.html#module-eject-the-cldr-adapter"},{"type":"function","title":"Routex.Extension.Cldr.configure/2","doc":"","ref":"Routex.Extension.Cldr.html#configure/2"},{"type":"function","title":"Routex.Extension.Cldr.display_name/2","doc":"","ref":"Routex.Extension.Cldr.html#display_name/2"},{"type":"module","title":"Routex.Extension.Cloak","doc":"Transforms routes to be unrecognizable.\n\n\n> #### Warning {: .warning}\n>\n> This extension is intended for testing and demonstration. It may change at\n> any given moment to generate other routes without prior notice.\n\n\nThe Cloak extension demonstrates how Routex enables extensions to transform\nroutes beyond recognition without breaking Phoenix' native and Routex' routing\nfeatures.\n\nCurrently it numbers all routes. Starting at 1 and incremening the counter for\neach route. It also shifts the parameter to the left; causing a chaotic route\nstructure.\n\nDo note: this still works with the Verified Routes extension. You can use the\noriginal, non transformed, routes in templates (e.g. `~p\"/products/%{product}\"`)\nand still be sure the transformed routes rendered at runtime (e.g. `/88/2` when product.id = 88)\nare valid routes.","ref":"Routex.Extension.Cloak.html"},{"type":"module","title":"Do (not) try this at home - Routex.Extension.Cloak","doc":"- Try this extension with a route generating extension like\n`Routex.Extension.Alternatives` for even more chaos.\n\n- Adapt this extension to use character repetition instead of numbers. Can you\nguess where `/90/**` leads to?","ref":"Routex.Extension.Cloak.html#module-do-not-try-this-at-home"},{"type":"module","title":"Configuration - Routex.Extension.Cloak","doc":"```diff\n# file /lib/example_web/routex_backend.ex\ndefmodule ExampleWeb.RoutexBackend do\n  use Routex.Backend,\n  extensions: [\n   Routex.Extension.AttrGetters, # required\n+  Routex.Extension.Cloak\n],\n```","ref":"Routex.Extension.Cloak.html#module-configuration"},{"type":"module","title":"Pseudo result - Routex.Extension.Cloak","doc":"Original                 Rewritten    Result (product_id: 88, 89, 90)\n    /products                ⇒     /1     ⇒    /1\n    /products/:id/edit       ⇒ /:id/2     ⇒ /88/2, /89/2, /90/2 etc...\n    /products/:id/show/edit  ⇒ /:id/3     ⇒ /88/3, /89/3, /90/3 etc...\n\n\n## `Routex.Attrs`\n**Requires**\n- none\n\n**Sets**\n- none","ref":"Routex.Extension.Cloak.html#module-pseudo-result"},{"type":"module","title":"Routex.Extension.Interpolation","doc":"A route may be defined with a routes `Routex.Attrs` interpolated\ninto it. These interpolations are specified using the usual `#{variable}`\ninterpolation syntax. Unlike some other routing solutions, interpolation\nis *not* restricted to the beginning of routes.\n\n> #### In combination with... {: .neutral}\n> Other extensions set `Routex.Attrs`. The attributes an extension sets is listed in it's documentation.\n> To define custom attributes for routes have a look at `Routex.Extension.Alternatives`\n>\n> When using `Routex.Extension.Alternatives` you might\n> want to disable auto prefixing for the whole Routex backend (see\n> `Routex.Extension.Alternatives`) or per route (see `Routex`).\n\n> #### Bare base route {: .warning}\n> The route as specified in the Router will be stripped from any\n> interpolation syntax. This allows you to use routes without interpolation\n> syntax in your templates (e.g. ~p\"/products\") and have them verified by\n> Verified Routes. The routes will be rendered with interpolated attributes\n> at run time.","ref":"Routex.Extension.Interpolation.html"},{"type":"module","title":"Configuration - Routex.Extension.Interpolation","doc":"none","ref":"Routex.Extension.Interpolation.html#module-configuration"},{"type":"module","title":"Usage - Routex.Extension.Interpolation","doc":"```elixir\n# file /lib/example_web/routes.ex\nlive \"/products/#{locale}/:id\", ProductLive.Index, :index\n```","ref":"Routex.Extension.Interpolation.html#module-usage"},{"type":"module","title":"Pseudo result - Routex.Extension.Interpolation","doc":"```elixir\n    # in combination with Routex.Extension.Alternatives with auto prefix\n    # disabled and 3 branches. It splits the routes and sets the :locale\n    # attribute which is used for interpolation.\n\n    Route                      Generated\n                               ⇒ /products/en/:id\n    /products/#{locale}/:id/   ⇒ /products/fr/:id\n                               ⇒ /products/fr/:id\n```\n\n## `Routex.Attrs`\n**Requires**\n- none\n\n**Sets**\n- none","ref":"Routex.Extension.Interpolation.html#module-pseudo-result"},{"type":"module","title":"Routex.Extension.RouteHelpers","doc":"Provides route helpers with support for automatic selecting alternatives\nroutes. The helpers can be used to override Phoenix' defaults as they are\na drop-in replacements.\n\nOnly use this extension when you make use of extensions generating alternative\nroutes, as otherwise the result will be the same as the official helpers.","ref":"Routex.Extension.RouteHelpers.html"},{"type":"module","title":"Configuration - Routex.Extension.RouteHelpers","doc":"```diff\n# file /lib/example_web/routex_backend.ex\ndefmodule ExampleWeb.RoutexBackend do\n  use Routex.Backend,\n  extensions: [\n    Routex.Extension.AttrGetters,  # required\n+   Routex.Extension.RouteHelpers\n],\n```\n\nPhoenix < 1.7 created an alias `Routes` by default. You can either replace it\nor add an alias for RoutexHelpers. Phoenix >= 1.7 deprecated the helpers\nin favor of Verified Routes.\n\nIn the example below we 'override' the default `Routes` alias to use\nRoutex' Route Helpers as a drop-in replacement, but keep the original helpers\nfunctions available by using alias `OriginalRoutes`.\n\n\n```diff\n# file /lib/example_web.ex\ndefp routex_helpers do\n\n+ alias ExampleWeb.Router.Helpers, as: OriginalRoutes\n+ alias ExampleWeb.Router.RoutexHelpers, as: Routes\n\nend\n```","ref":"Routex.Extension.RouteHelpers.html#module-configuration"},{"type":"module","title":"Pseudo result - Routex.Extension.RouteHelpers","doc":"# When alternatives are created it uses auto-selection to keep the user 'in branch'.\n\n    # in (h)eex template\n     Product #1 \n\n    # is replaced during during compilation with:\n    case alternative do\n       nil ⇒  \"/products/#{product}\"\n      \"en\" ⇒  \"/products/#{product}\"\n      \"nl\" ⇒  \"/europe/nl/products/#{product}\"\n      \"be\" ⇒  \"/europe/be/products/#{product}\"\n    end\n\n## `Routex.Attrs`\n**Requires**\n- none\n\n**Sets**\n- none","ref":"Routex.Extension.RouteHelpers.html#module-pseudo-result"},{"type":"function","title":"Routex.Extension.RouteHelpers.build_case/1","doc":"","ref":"Routex.Extension.RouteHelpers.html#build_case/1"},{"type":"function","title":"Routex.Extension.RouteHelpers.build_case_clauses/4","doc":"","ref":"Routex.Extension.RouteHelpers.html#build_case_clauses/4"},{"type":"function","title":"Routex.Extension.RouteHelpers.dynamic_delegate_with_arity/3","doc":"","ref":"Routex.Extension.RouteHelpers.html#dynamic_delegate_with_arity/3"},{"type":"function","title":"Routex.Extension.RouteHelpers.dynamic_fn_with_arity/3","doc":"","ref":"Routex.Extension.RouteHelpers.html#dynamic_fn_with_arity/3"},{"type":"module","title":"Routex.Extension.Translations","doc":"Enables users to enter URLs using localized terms which can enhance user engagement\nand content relevance.\n\nExtracts segments of a routes' path to a translations domain file (default: `routes.po`)\nfor translation. At compile-time it combines the translated segments to transform routes.\n\nThis extension expects either a `:language` attribute or a `:locale` attribute. When only\n`:locale` is provided it will try to extract the language from the locale tag. This algorithm\ncovers Alpha-2 and Alpha-3 codes (see:\n[ISO](https://datatracker.ietf.org/doc/html/rfc5646#section-2.2.1))\n\n> #### In combination with... {: .neutral}\n> How to combine this extension for localization is written in de [Localization Guide](guides/LOCALIZED_ROUTES.md)","ref":"Routex.Extension.Translations.html"},{"type":"module","title":"Configuration - Routex.Extension.Translations","doc":"```diff\ndefmodule ExampleWeb.RoutexBackend do\nuse Routex.Backend,\nextensions: [\n  Routex.Extension.AttrGetters, # required\n+ Routex.Extension.Translations\n]\n+ translations_backend: MyApp.Gettext,\n+ translations_domain: \"routes\",\n```","ref":"Routex.Extension.Translations.html#module-configuration"},{"type":"module","title":"Pseudo result - Routex.Extension.Translations","doc":"# when translated to Spanish in the .po file\n    # - products: producto\n    # - edit: editar\n\n    /products/:id/edit  ⇒ /producto/:id/editar\n\n## `Routex.Attrs`\n**Requires**\n- language || locale\n\n**Sets**\n- none","ref":"Routex.Extension.Translations.html#module-pseudo-result"},{"type":"module","title":"Use case(s) - Routex.Extension.Translations","doc":"This extension can be combined with `Routex.Extension.Alternatives` to create\nmultilingual routes.\n\nUse Alternatives to create new branches and provide a `:language` or `:locale` per branch and\nTranslations to translate the alternative routes.\n\n                        ⇒ /products/:id/edit                  language: \"en\"\n    /products/:id/edit  ⇒ /nederland/producten/:id/bewerken   language: \"nl\"\n                        ⇒ /espana/producto/:id/editar         language: \"es\"","ref":"Routex.Extension.Translations.html#module-use-case-s"},{"type":"module","title":"Routex.Extension.VerifiedRoutes","doc":"Supports the use of original route paths in controllers and templates while rendering\ntransformed route paths at runtime without performance impact.\n\n> #### Implementation summary {:.info}\n> Each sigil and function eventualy delegates to the official\n> `Phoenix.VerifiedRoutes`.  If a non-branching route is provided it will\n> simply delegate to the official Phoenix function. If a branching route is\n> provided, it will use a branching mechanism before delegating.\n\n#### Alternative Verified Route sigil\nProvides a sigil (default: `~l`) to verify transformed and/or branching routes.\nThe sigil to use can be set to `~p` to override the default of Phoenix as\nit is a drop-in replacement. If you choose to override the default Phoenix sigil,\nit is renamed (default: `~o`) and can be used when unaltered behavior is required.\n\n#### Variants of url/{2,3,4} and path/{2,3}\nProvides branching variants of (and delegates to) macro's provided by\n`Phoenix.VerifiedRoutes`. Both new macro's detect whether branching should be\napplied.","ref":"Routex.Extension.VerifiedRoutes.html"},{"type":"module","title":"Options - Routex.Extension.VerifiedRoutes","doc":"- `verified_sigil_routex`: Sigil to use for Routex verified routes (default `\"~l\"`)\n- `verified_sigil_phoenix`: Replacement for the native (original) sigil when `verified_sigil_routex`\n  is set to \"~p\". (default: `\"~o\"`)\n - `verified_url_routex`: Function name to use for Routex verified routes powered `url`. (default: `:rtx_url`)\n- `verified_url_phoenix`: Replacement for the native `url` function when `verified_url_routex`\n  is set to `:url`. (default: `:phx_url`)\n - `verified_path_routex`: Function name to use for Routex verified routes powered `path` (default `:rtx_path`)\n- `verified_path_phoenix`: Replacement for the native `path` function  when `verified_path_routex`\n  is set to `:path`. (default: `:phx_path`)\n\nWhen `verified_sigil_routex` is set to \"~p\" an additional change must be made.\n\n```diff\n# file /lib/example_web.ex\ndefp routex_helpers do\n+  import Phoenix.VerifiedRoutes,\n+      except: [sigil_p: 2, url: 1, url: 2, url: 3, path: 2, path: 3]\n\n    import unquote(__MODULE__).Router.RoutexHelpers, only: :macros\n    alias unquote(__MODULE__).Router.RoutexHelpers, as: Routes\nend\n```","ref":"Routex.Extension.VerifiedRoutes.html#module-options"},{"type":"module","title":"Configuration - Routex.Extension.VerifiedRoutes","doc":"```diff\n# file /lib/example_web/routex_backend.ex\ndefmodule ExampleWeb.RoutexBackend do\n  use Routex.Backend,\n  extensions: [\n    Routex.Extension.AttrGetters, # required\n    Routex.Extension.Alternatives,\n    [...]\n+   Routex.Extension.VerifiedRoutes\n],\n+ verified_sigil_routex: \"~p\",\n+ verified_sigil_phoenix: \"~o\",\n+ verified_url_routex: :url,\n+ verified_url_phoenix: :url_native,\n+ verified_path_routex: :path,\n+ verified_path_phoenix: :path_native,\n```","ref":"Routex.Extension.VerifiedRoutes.html#module-configuration"},{"type":"module","title":"Pseudo result - Routex.Extension.VerifiedRoutes","doc":"```elixir\n# given Routex behavior is assigned ~l\n# given the default behavior is assigned ~o\n# given the official macro of Phoenix is assigned ~p\n\n# given another extension has transformed the route\n~o\"/products/#{product}\"   ⇒  ~p\"/products/#{products}\"\n~l\"/products/#{product}\"   ⇒  ~p\"/transformed/products/#{product}\"\n\n# given another extension has generated branches / alternative routes\n~o\"/products/#{product}\"  ⇒  ~p\"/products/#{products}\"\n~l\"/products/#{product}\"  ⇒\n        case current_branch do\n          nil     ⇒  ~p\"/products/#{product}\"\n          \"en\"    ⇒  ~p\"/products/en/#{product}\"\n          \"eu_nl\" ⇒  ~p\"/europe/nl/products/#{product}\"\n          \"eu_be\" ⇒  ~p\"/europe/be/products/#{product}\"\n        end\n```\n\n## `Routex.Attrs`\n**Requires**\n- none\n\n**Sets**\n- none","ref":"Routex.Extension.VerifiedRoutes.html#module-pseudo-result"},{"type":"exception","title":"Routex.Extension.Interpolation.NonUniqError","doc":"Raised when a list of routes contains routes with the same path and verb.\n\n```elixir\n[%Route{\n  path: \"/foo\"\n  verb: :get},\n%Route{\n  path: \"/foo\"\n  verb: :post}, # <-- different\n%Route{\n  path: \"/foo\"\n  verb: :get} # <-- duplicate\n]\n```\n\nSolution: use a combination of interpolated attributes that form a unique set.","ref":"Routex.Extension.Interpolation.NonUniqError.html"},{"type":"module","title":"Routex.Extension.Alternatives.Branch.Flat","doc":"Struct for flattened branch","ref":"Routex.Extension.Alternatives.Branch.Flat.html"},{"type":"type","title":"Routex.Extension.Alternatives.Branch.Flat.t/0","doc":"","ref":"Routex.Extension.Alternatives.Branch.Flat.html#t:t/0"},{"type":"module","title":"Routex.Extension.Alternatives.Branch.Nested","doc":"Struct for branch with optionally nested branches","ref":"Routex.Extension.Alternatives.Branch.Nested.html"},{"type":"type","title":"Routex.Extension.Alternatives.Branch.Nested.t/0","doc":"","ref":"Routex.Extension.Alternatives.Branch.Nested.html#t:t/0"},{"type":"module","title":"Routex.Extension.Alternatives.Config","doc":"Module to create and validate a Config struct","ref":"Routex.Extension.Alternatives.Config.html"},{"type":"type","title":"Routex.Extension.Alternatives.Config.t/0","doc":"","ref":"Routex.Extension.Alternatives.Config.html#t:t/0"},{"type":"module","title":"Routex.Extension.Alternatives.Exceptions","doc":"","ref":"Routex.Extension.Alternatives.Exceptions.html"},{"type":"exception","title":"Routex.Extension.Alternatives.Exceptions.AttrsMismatchError","doc":"Raised when the custom attributes of branches do not have the same keys.\n\n```elixir\n%{\n  branches: %{\n    \"/\"      => %{attrs: %{key1: 1, key2: 2}},\n    \"/other\" => %{attrs: %{key1: 1}} # missing :key2\n  }\n}\n```\n\nTo fix this, make the attribute maps consistent or use an attributes struct.","ref":"Routex.Extension.Alternatives.Exceptions.AttrsMismatchError.html"},{"type":"exception","title":"Routex.Extension.Alternatives.Exceptions.MissingRootSlugError","doc":"Raised when the branch map does not start with the root branch \"/\".\n\n```elixir\n%{\n  branches: %{\n    \"/first\"  =>    %{attrs: %{key1: 1}},\n    \"/other\"  =>    %{attrs: %{key1: 1}}},\n}\n```\n\nTo fix this, include a branch for the root \"/\".\n\n```elixir\n`%{\n  branches: %{\n    \"/\" => %{\n      attrs: %{level: 1}\n      branches: %{\n        \"/first\"  =>    %{attrs: %{level: 2}},\n        \"/other\"  =>    %{attrs: %{level: 2}}\n      }\n    },\n  }\n}\n```","ref":"Routex.Extension.Alternatives.Exceptions.MissingRootSlugError.html"},{"type":"extras","title":"Overview","doc":"# \n![Logo of Routex](assets/logo_horizontal.png \"Routex Logo\")\n\n![Coveralls](https://img.shields.io/coveralls/github/BartOtten/routex)\n[![Build Status](https://github.com/BartOtten/routex/actions/workflows/elixir.yml/badge.svg?event=push)](https://github.com/BartOtten/routex/actions/workflows/elixir.yml)\n[![Last Updated](https://img.shields.io/github/last-commit/BartOtten/routex.svg)](https://github.com/BartOtten/routex/commits/main)\n[![Hex.pm](https://img.shields.io/hexpm/v/routex)](https://hex.pm/packages/routex)\n![Hex.pm](https://img.shields.io/hexpm/l/routex)\n\n# Routex: Supercharge your Phoenix Router\n\nThis powerful library works together with Phoenix Router to provide the ultimate\nrouting solution. It simplifies route manipulation, giving you the control you\nneed to handle even the most demanding routing requirements.\n\nRoutex comes with extensions for internationalization (i18n), localization\n(l10n), translated (multilingual) URLs, alternative routes generation and [many\nmore](docs/EXTENSION_SUMMARIES.md). Its modern extensible architecture enables you to easily\nbuild custom solutions that work harmoniously with other extensions.","ref":"readme.html"},{"type":"extras","title":"Top Features and Benefits - Overview","doc":"* **No dependencies, no state**: Routex is unique in not requiring any external dependency\n  and works *by default* without proces state.\n\n* **Powerful transformations**: Routex supports advanced route transformations, including\n  everything needed for internationalization (i18n) and localization (l10n).\n\n* **Extension driven**: Being extension driven, Routex can be adapted to your\n  specific needs. It's architecture allows you to write your own routing\n  features without having to worry about breaking existing functionality. Routex\n  ships with extensions covering a wide range of use cases.\n  Have a look at [a summary of extensions](docs/EXTENSION_SUMMARIES.md).\n\n* **Optimized for performance**: Built to fit between route configuration and\n  route compilation. Routex enhances Phoenix routing without adding runtime\n  overhead, ensuring that applications run as fast as ever.\n\n* **Detailed documentation**: Comprehensive, well-organized documentation\n  provides clear guidance on installation, configuration, and best practices,\n  making Routex approachable for developers at all levels. For example: If you\n  are interested in internationalization (i18n) or localization (l10n) have a\n  look at the [Localized Routes Guide](docs/guides/LOCALIZED_ROUTES.md).","ref":"readme.html#top-features-and-benefits"},{"type":"extras","title":"Installation and usage - Overview","doc":"**[Usage Guide](USAGE.md)** - requirements and installation.\ninstructions.\n\n**[Documentation](https://hexdocs.pm/routex)** - from step-by-step guides till in-depth explanations.","ref":"readme.html#installation-and-usage"},{"type":"extras","title":"Give it a try! - Overview","doc":"**[Online demo](https://routex.fly.dev/)** - have a look or get the\n[code](https://github.com/BartOtten/routex_example/).","ref":"readme.html#give-it-a-try"},{"type":"extras","title":"Articles - Overview","doc":"To help you understand where Routex fits in.\n\n### [How Routex and Phoenix Router Work Together](docs/ROUTEX_AND_PHOENIX_ROUTER.md)\nUnderstanding how Routex, its extensions, and Phoenix Router work together can\nbe tricky at first sight. To help you understand, we came up with an analogy.\n\n### [Routex compared to Phoenix Router and Cldr Routes](docs/COMPARISON.md)\nWe published a comparison with the intended to help you understand\nthe differences, strengths, and tradeoffs when deciding which routing solution\nbest fits your needs.","ref":"readme.html#articles"},{"type":"extras","title":"Included extensions","doc":"# Routex Extensions\n\nRoutex relies on extensions to provide features. Each extension provides a\nsingle feature and should minimize hard dependencies on other extensions.\nInstead, Routex advises to make use of `Routex.Attrs` to share attributes;\nallowing extensions to work together without being coupled.\n\nThe documentation of each extension lists any provided or required\n`Routex.Attrs`.","ref":"extension_summaries.html"},{"type":"extras","title":"Alternatives - Included extensions","doc":"Create alternative routes based on `branches` configured in a Routex backend\nmodule. Branches can be nested and each branch can provide it's own attributes to\nbe shared with other extensions.\n\n[Alternatives Documentation](https://hexdocs.pm/routex/Routex.Extension.Alternatives.html)","ref":"extension_summaries.html#alternatives"},{"type":"extras","title":"Translations - Included extensions","doc":"This extension extracts segments of a routes' path to a `routes.po` file for\ntranslation. At compile-time it combines the translated segments to translate\nroutes. As a result, users can enter URLs using localized terms which can\nenhance user engagement and content relevance.\n\n[Translations Documentation](https://hexdocs.pm/routex/Routex.Extension.Translations.html)","ref":"extension_summaries.html#translations"},{"type":"extras","title":"Interpolation - Included extensions","doc":"With this extension enabled, *any* attribute assigned to a route can be used\nfor route interpolation. Most effective with an extension which enables\nalternative routes generation (such as extension `Alternatives`).\n\n    /product/#{territory}/:id/#{language}  => /product/europe/:id/nl\n\n[Interpolation Documentation](https://hexdocs.pm/routex/Routex.Extension.Interpolation.html)\n\n>","ref":"extension_summaries.html#interpolation"},{"type":"extras","title":"Multilingual Routes {: .neutral} - Included extensions","doc":"> The Alternatives extension can be combined with the Translations and Interpolation\n> extension to create multilingual routes. The Alternatives extension can provide the `:locale`\n> attribute used by the Translations extension and other attributes to use with the Interpolation\n> extension.\n>\n> Guide: [Localized Routes with Routex](guides/LOCALIZED_ROUTES.md)\n>\n> \t  Original            Step 1: Alternatives                Step 2: Translations\n> \t\t\t\t\t\t  ⇒ /products/:id/edit                ⇒ /products/:id/edit\n> \t  /products/:id/edit  ⇒ /eu/netherlands/products/:id/edit ⇒ /eu/nederland/producten/:id/bewerken\n> \t\t\t\t\t\t  ⇒ /eu/spain/products/:id/edit\t      ⇒ /eu/espana/producto/:id/editar\n> \t\t\t\t\t\t  ⇒ /gb/products/:id/edit             ⇒ /gb/products/:id/edit\n>","ref":"extension_summaries.html#multilingual-routes-neutral"},{"type":"extras","title":"Alternative Getters - Included extensions","doc":"Creates a helper function `alternatives/1` to get a list of alternative slugs\nand their route attributes.  The current route is also included and has attribute\n`current?: true`. As `Routex` sets the `@url` assign you can simply\nget all routes to the current page with `alternatives(@url)`, use an attribute in the\nroute as button text and highlight the current active route button.\n\n[Alternative Getters Documentation](https://hexdocs.pm/routex/Routex.Extension.AlternativeGetters.html)","ref":"extension_summaries.html#alternative-getters"},{"type":"extras","title":"Verified Routes - Included extensions","doc":"Routex is fully compatible with Verified Routes.\n\nThis extension creates a sigil (default: `~l`) with the ability to branch based\non the current alternative branch of a user. It is able to verify routes even\nwhen thy have been transformed by Routex extensions. Optionally this sigil can\nbe set to `~p` (Phoenix' default) as it is a drop-in replacement.\n\nIt also provides branching variants of `url/{2,3,4}` and `path/{2,3}`.\n\n[Verified Routes Documentation](https://hexdocs.pm/routex/Routex.Extension.VerifiedRoutes.html)","ref":"extension_summaries.html#verified-routes"},{"type":"extras","title":"Assigns - Included extensions","doc":"With this extension you can add (a subset of) attributes set by other extensions\nto Phoenix' assigns making them available in components and controllers with the\n`@` assigns operator (optionally under a namespace)\n\n    @namespace.area      =>  :eu_nl\n\t@namespace.contact   =>  \"contact@example.com\"\n\n[Assigns Documentation](https://hexdocs.pm/routex/Routex.Extension.Assigns.html)","ref":"extension_summaries.html#assigns"},{"type":"extras","title":"Attribute Getters - Included extensions","doc":"Creates a helper function `attrs/1` to get all `Routex.Attrs` of a route. As\n`Routex` sets the `@url` assign you can simply get all attributes for the\ncurrent page with `attrs(@url)`.\n\nThe benefit over `assigns` is not loading the attributes into the process memory,\nbut stil be able to easily access them when needed.\n\n*Tip*: use Assigns for attributes always required and Attribute Getters for\nthose that are conditionally.\n\n\n[Attribute Getters Documentation](https://hexdocs.pm/routex/Routex.Extension.AttrGetters.html)","ref":"extension_summaries.html#attribute-getters"},{"type":"extras","title":"Cldr Adapter - Included extensions","doc":"Adapter for projects using :ex_cldr.\n\n[Cldr Adapter Documentation](https://hexdocs.pm/routex/Routex.Extension.Cldr.html)","ref":"extension_summaries.html#cldr-adapter"},{"type":"extras","title":"Route Helpers - Included extensions","doc":"Creates Phoenix Helpers that have the ability to branch based on the current\nalternative branch of a user. Optionally these helpers can replace the original\nPhoenix Route Helpers as they are drop-ins.\n\n[Route Helpers Documentation](https://hexdocs.pm/routex/Routex.Extension.RouteHelpers.html)","ref":"extension_summaries.html#route-helpers"},{"type":"extras","title":"Cloak (show case) - Included extensions","doc":"Transforms routes to be unrecognizable. This extension is a show case and may\nchange at any given moment to generate other routes without prior notice.\n\nIn this example it numbers all routes starting at 01 and increments the counter\nfor each route. It also shifts the parameter to the left; causing a chaotic\nroute structure. Do note: this still works with the Verified Routes extension\nwhile using the standard route (e.g. `<.link navigate={~p\"/products\">`) in\ntemplates.\n\n\n      Original                 Rewritten     Result (product_id: 88, 89, 90)\n      /products                ⇒     /01     ⇒    /01\n      /products/:id/edit       ⇒ /:id/02     ⇒ /88/02, /89/02, /90/02 etc...\n      /products/:id/show/edit  ⇒ /:id/03     ⇒ /88/03, /89/03, /90/03 etc...\n\n\n[Cloak Documentation](https://hexdocs.pm/routex/Routex.Extension.Cloak.html)","ref":"extension_summaries.html#cloak-show-case"},{"type":"extras","title":"Getting started","doc":"# Usage","ref":"usage.html"},{"type":"extras","title":"Requirements - Getting started","doc":"- Elixir >=1.11\n- Phoenix >= 1.6.0\n- Phoenix LiveView >= 0.16 (optional)","ref":"usage.html#requirements"},{"type":"extras","title":"Installation - Getting started","doc":"You can install this library by adding it to your list of dependencies in `mix.exs`. (use `mix hex.info routex` to find the latest version):\n\n```diff\ndef deps do\n  [\n     ...other deps\n+    {:routex, \">= 0.0.0\"}\n  ]\nend\n```\n\nModify the entrypoint of your web interface definition.\n```diff\n# file: lib/example_web.ex\n\n+  use Routex.Router  # always before Phoenix Router\n   use Phoenix.Router, helpers: false\n\n# in controller\n   unquote(verified_routes())\n+  unquote(routex_helpers())\n\n# in live_view\n      unquote(html_helpers())\n+     on_mount(unquote(__MODULE__).Router.RoutexHelpers)\n\n# in view_helpers or html_helpers\n   unquote(verified_routes())\n+  unquote(routex_helpers())\n\n# insert new private function\n+  defp routex_helpers do\n+    quote do\n+      import unquote(__MODULE__).Router.RoutexHelpers, only: :macros\n+      alias unquote(__MODULE__).Router.RoutexHelpers, as: Routes\n+    end\n+  end\n```","ref":"usage.html#installation"},{"type":"extras","title":"About the on_mount hook - Getting started","doc":"The `on_mount` hook is generated by Routex in a module created during compilation.\nIt attaches a `handle_param` which in turn assigns a few (helper) values\nto the connection and/or socket. This includes the current url and assigns added\nby Routex Extensions.","ref":"usage.html#about-the-on_mount-hook"},{"type":"extras","title":"Configuration - Getting started","doc":"To use `Routex`, a module that calls `use Routex.Backend` (referred to below as a\n\"backend\") has to be defined. It includes a list with extensions and\nconfiguration of extensions.\n\n```elixir\ndefmodule ExampleWeb.RoutexBackend do\nuse Routex.Backend,\n  extensions: [\n  Routex.Extension.AttrGetters,  # required\n  # ...other extensions...\n  ],\nend\n```","ref":"usage.html#configuration"},{"type":"extras","title":"Extensions - Getting started","doc":"Routex is merely a framework and relies on extensions to provide features. Each\nextension provides a single feature. The extensions have their own documentation\nwhich specifies how to configure and use them.","ref":"usage.html#extensions"},{"type":"extras","title":"Preprocess routes with Routex - Getting started","doc":"`Routex` will preprocess any route wrapped in a `preprocess_using` block; either\ndirect or indirect. It uses the backend passed as the first argument.  This\nallows the use of multiple backends (e.g. to use different extensions for admin\nroutes)\n\n```diff\n# file: router.ex\n  scope \"/\", ExampleWeb, host: \"admin.\", as: :admin do\n    pipe_through :browser\n\n+    preprocess_using ExampleWeb.RoutexBackendAdmin do\n      # [...routes...]\n+    end\n  end\n\n+ preprocess_using ExampleWeb.RoutexBackend do\n    scope \"/\", ExampleWeb do\n      pipe_through [:browser, :redirect_if_user_is_authenticated]\n      # [...routes...]\n    end\n\n    scope \"/\", ExampleWeb do\n      pipe_through [:browser, :require_authenticated_user]\n      # [...routes...]\n    end\n+ end\n```\n\nWhen you run into issues, please have a look at the [Troubleshooting](docs/TROUBLESHOOTING.md)","ref":"usage.html#preprocess-routes-with-routex"},{"type":"extras","title":"Localized Routes","doc":"# Localized Routes with Routex\n\nA core feature of Routex is to enable Localized Routes in Phoenix. Optionally with\ntranslated URLs, enhancing user engagement and content relevance.\n\nIn this tutorial we will explain how multiple extensions are combined to...\n1. have a product page with regional URL's\n2. (optional) use translated routes\n2. using automatically localized verified routes\n3. display links to other locales\n\nAll without changing a single route in your templates!\n\n\n                        ⇒ /products/:id/edit                    @loc.locale = \"en_US\"\n    /products/:id/edit  ⇒ /eu/nederland/producten/:id/bewerken  @loc.locale = \"nl_NL\"\n                        ⇒ /eu/espana/producto/:id/editar        @loc.locale = \"es_ES\"\n                        ⇒ /gb/products/:id/edit                 @loc.locale = \"en_GB\"\n\n\nThis tutorial assumes you have followed the [usage guide](USAGE.md) to setup\nRoutex.\n\nIf you encounter any issues with Routex or this tutorial, feel free to [open a topic at Elixir\nForums](https://elixirforum.com/tag/routex) or create an issue at GitHub.","ref":"localized_routes.html"},{"type":"extras","title":"What we start with - Localized Routes","doc":"This tutorial uses an example Router with multiple routes to the product page.\nThe route.ex file contains something like the example below.\n\n      preprocess_using ExampleWeb.RoutexBackend do\n        scope \"/\", ExampleWeb do\n          pipe_through :browser\n\n          live \"/products\", ProductLive.Index, :index\n          live \"/products/new\", ProductLive.Index, :new\n          live \"/products/:id/edit\", ProductLive.Index, :edit\n          live \"/products/:id\", ProductLive.Show, :show\n          live \"/products/:id/show/edit\", ProductLive.Show, :edit\n        end\n      end\n\nWhen you run `mix phx.routes` you will see those routes as:\n\n    product_show_path  GET     /products/:id/show/edit                ExampleWeb.ProductLive.Show :edit\n    product_show_path  GET     /products/:id                          ExampleWeb.ProductLive.Show :show\n    product_index_path  GET    /products/:id/edit                     ExampleWeb.ProductLive.Index :edit\n    product_index_path  GET    /products/new                          ExampleWeb.ProductLive.Index :new\n    product_index_path  GET    /products                              ExampleWeb.ProductLive.Index :index\n\nYou want these pages to be accessible from multiple (translated) URLs.","ref":"localized_routes.html#what-we-start-with"},{"type":"extras","title":"Step 1: Generate alternative URLs - Localized Routes","doc":"The `Routex.Extension.Alternatives` generates alternative routes. Add it to the list of extensions\nand provide a minimal configuration.\n\n```diff\nuse Routex.Backend,\nextensions: [\n  Routex.Extension.AttrGetters, # required\n+ Routex.Extension.Alternatives,\n],\n+ alternatives: %{\n+      \"/\" => %{\n+        branches: %{\n+          \"/europe\" => %{\n+              branches: %{\n+                \"/nl\" => %{},\n+                \"/be\" => %{}\n+            }\n+          },\n+          \"/gb\" => %{}\n+        }\n+      }\n+    }\n```\n\nYou can confirm it works by running `mix phx.routes`. It now shows a lot more\nroutes as alternatives are generated for each route within the\n`preprocess_using` block. For example the route to `/products/:id/show/edit` has\nmultiple alternatives.\n\n```\nproduct_show_path           GET     /products/:id/show/edit               ExampleWeb.ProductLive.Show :edit\nproduct_show_europe_path    GET     /europe/products/:id/show/edit        ExampleWeb.ProductLive.Show :edit\nproduct_show_europe_be_path GET     /europe/be/products/:id/show/edit     ExampleWeb.ProductLive.Show :edit\nproduct_show_europe_nl_path GET     /europe/nl/products/:id/show/edit     ExampleWeb.ProductLive.Show :edit\nproduct_show_gb_path        GET     /gb/products/:id/show/edit            ExampleWeb.ProductLive.Show :edit\n```\n\nAs you can see the routes are still in the English language; we need another extension to\ntranslate them\n\n\n## (optional) Step 2: Translate the alternative routes\n\nThe `Routex.Extension.Translation` makes routes translatable by splitting the route\ninto segments (e.g. `[\"products\", \"show\", \"edit\"]`) and extracting these\nsegments to a `routes.po` file for translation. You might recognize the `.po`\nextension from your Phoenix project; it's the extension used by Gettext. Gettext\nis a standard for i18n in different communities, meaning there is a great set of\ntooling for developers and translators. This also means your routes segments can be\ntranslated with the same tooling as used for all other translations in Phoenix!\n\nAdd the extension and it's minimal configuration.\n\n```diff\nuse Routex.Backend,\nextensions: [\n  Routex.Extension.AttrGetters, # required\n  Routex.Extension.Alternatives,\n+ Routex.Extension.Translations,\n],\n  alternatives: %{...},\n+ translations_backend: ExampleWeb.Gettext,\n```\n\nAs Routex need to know which translation to use for what route, we need to set\nan attribute `:locale` or `:language` per alternative.\n\nLuckily this is covered by `Extension.Alternatives` as it supports setting the\n`:attrs` key per branch. Let's extend the alternatives configuration with by\nsetting the `:locale` attribute. While we are add it, we also give the branches\na `:display_name` attribute.\n\n```diff\n alternatives: %{\n      \"/\" => %{\n+        attrs: %{locale: \"en-150\", display_name: \"Global\"},\n         branches: %{\n           \"/europe\" => %{\n+              attrs: %{locale: \"en-150\", display_name: \"Europe\"},\n               branches: %{\n+                \"/nl\" => %{attrs: %{locale: \"nl_NL\", display_name: \"The Netherlands\"}},\n+                \"/be\" => %{attrs: %{locale: \"nl_BE\", display_name: \"Belgium\"}}\n             }\n           },\n+          \"/gb\" => %{attrs: %{locale: \"en-150\", display_name: \"Great Britain\"}}\n         }\n       }\n     }\n```\n\nIf this is the first time you add translations in your project, you need to\ngenerate the folder structure which Gettext can use to detect languages to\ntranslate to. We need two languages in this tutorial: 'en' and 'nl'. As 'en' is\nthe default for routes we only need to create translations for 'nl'.\n\n```\nmix gettext.extract\nmix gettext.merge priv/gettext --locale nl\n```\n\nYou should see a message that Gettext has generated new translation files which\ncan be found in the `priv/gettext/nl` folder\n\n```\npriv/\n gettext/\n   nl/\n     LC_MESSAGES/\n       default.po # phoenix translations\n       routes.po  # routex translations\n```\n\nNow you can translate the segments by opening the `routes.po` file with your\nfavorite `.po` editor. Here are a few suggestions:\n\n* `GNU Emacs (with po-mode)`: Linux, MacOSX, and Windows.\n* `Lokalize`: runs on KDE desktop for Linux (replacement for KBabel; formerly known as KAider)\n* `Poedit`: Linux, MacOSX, and Windows\n* `OmegaT`: Linux, MacOSX, and Windows\n* `Vim`: Linux, MacOSX, and Windows with PO ftplugin for easier editing of GNU gettext PO files.\n* `gted plugin for Eclipse`: (if you are already using Eclipse)\n* `gtranslator`: Linux/Gnome\n* `Virtaal`: Windows, Mac (Beta version)\n\nOnce you have translated the route segments, list all routes using `mix\nphx.routes`. You will see some routes have been translated. We are getting\nthere!\n\n```\nproduct_show_path            GET     /products/:id/show/edit\nproduct_show_europe_path     GET     /europe/products/:id/show/edit\nproduct_show_europe_be_path  GET     /europe/be/producten/:id/toon/bewerken\nproduct_show_europe_nl_path  GET     /europe/nl/producten/:id/toon/bewerken\n```\n\nNow we have the routes it would be nice if users stay within their `locale`\nwhile browsing pages.","ref":"localized_routes.html#step-1-generate-alternative-urls"},{"type":"extras","title":"Step 3: Dynamic links in your application - Localized Routes","doc":"When you start your app with `mix phx.server` and you visit a 'localized' page\nsuch as `/europe/nl/producten`, you will notice that every link on the page will\nbring you back to the non-locale route. In the code the path of the link is written\nlike `~p\"/products\"`. It would be nice if instead of always rendering a link to\n`/products`, Phoenix would instead render a localized link. This is done by\n`Routex.Extension.VerifiedRoutes`.\n\n> **Note**\n> In older Phoenix applications you might find something like\n> `ExampleAppWeb.Router.Helpers.product_path(conn_or_endpoint, :show, \"hello\")`.\n> These are Phoenix Router Helpers and those are deprecated in favor of the\n> Verified Routes using `~p\"/my_path\"`. When you can't migrate, you can use\n> `Routex.Extension.RouteHelpers` instead of `Routex.Extension.VerifiedRoutes`.\n\nYou might already have guessed it: we are gonna add the extension and some\nconfiguration to the backend.\n\n```diff\nuse Routex.Backend,\nextensions: [\n  Routex.Extension.AttrGetters, # required\n  Routex.Extension.Alternatives,\n  Routex.Extension.Translations,\n+ Routex.Extension.VerifiedRoutes,\n],\n  alternatives: %{...},\n  translations_backend: ExampleWeb.Gettext,\n+ verified_sigil_routex: \"~p\",\n+ verified_sigil_phoenix: \"~o\",\n+ verified_url_routex: :url,\n+ verified_path_routex: :path\n```\n\nBy default the extension uses non-standard macro names. As we want to have\ndynamic routes throughout our application, we choose to override the names used\nby Phoenix in your application and rename the originals. This way you do not\nneed to modify all your templates. Convenient.\n\nTo not have duplicated imports, add this to your routex_helpers in `example_web.ex`\n\n```diff\n  def routex_helpers do\n    quote do\n+      import Phoenix.VerifiedRoutes,\n+        except: [sigil_p: 2, url: 1, url: 2, url: 3, path: 2, path: 3]\n\n      import unquote(__MODULE__).Router.RoutexHelpers, only: :macros\n      alias unquote(__MODULE__).Router.RoutexHelpers, as: Routes\n    end\n  end\n```\n\nNow when you start your app with `mix phx.server` you will notice an explanation\nis printed about the usage of Routex Verified Routes. This informs other\ndevelopers of the overrides.\n\n```\nDue to the configuration in module `ExampleWeb.RoutexBackend` one or multiple\nRoutex variants use the default name of their native Phoenix equivalents. The native\nmacro's, sigils or functions have been renamed.\n\n Native              | Routex\n -----------------------------------------\n ~o                  | ~p\n url_phx             | url\n path_phx            | path\n\n Documentation: https://hexdocs.pm/routex/extensions/verified_routes.html\n ```\n\n\nWhen you visit a 'localized' page such as `/europe/nl/producten` you will notice\nthat every link on the page will keep you within the localized environment\n`/europe/nl/`. Keeping users in a localized environment is great, but giving\nthem an option to switch to another locale would be even better.\n\nLet's empower our visitors!","ref":"localized_routes.html#step-3-dynamic-links-in-your-application"},{"type":"extras","title":"Step 4: Show alternative pages to the user - Localized Routes","doc":"The `Routex.Extension.AlternativeGetters` generates function at compile time to\ndynamically fetch alternative routes for the current url without overhead. Let's\nonce again add the extension.\n\n```diff\nuse Routex.Backend,\nextensions: [\n  Routex.Extension.AttrGetters, # required\n  Routex.Extension.Alternatives,\n  Routex.Extension.Translations,\n  Routex.Extension.VerifiedRoutes,\n+ Routex.Extension.AlternativeGetters,\n],\n  alternatives: %{...},\n  translations_backend: ExampleWeb.Gettext,\n  verified_sigil_routex: \"~p\",\n  verified_sigil_phoenix: \"~o\",\n  verified_url_routex: :url,\n  verified_path_routex: :path\n```\n\nAll created functions pattern match on a given URL and return the alternatives\nwith a `slug`, the `match?` attribute which is true if the route pattern matches\nthe provided path, and `attributes` of the route.\n\n```\niex> ExampleWeb.Router.RoutexHelpers.alternatives(\"https://example.com/products?search=bar#top\")\n[\n  %Routex.Extension.AlternativeGetters{\n    slug: \"/products?search=bar#top\",\n    attrs: %{\n      name: \"Worldwide\",\n      locale: \"en-US\",\n      [...]\n    },\n    match?: true\n  },\n  %Routex.Extension.AlternativeGetters{\n    slug: \"/europe/products?search=bar#top\",\n    attrs: %{\n      name: \"Europe\",\n      locale: \"en-150\",\n      [...]\n    },\n    match?: false\n  },\n  %Routex.Extension.AlternativeGetters{\n    slug: \"/europe/be/producten?search=bar#top\",\n    attrs: %{\n      name: \"Belgium\",\n      locale: \"nl-BE\",\n       [...]\n    },\n    match?: false\n  },\n  [...]\n```\n\nAs Routex automatically assigns the current url to `@url` we have all\ningredients instantly available in our templates! It becomes a matter of looping\nover the results to generate links.\n\n```heex\n <!-- alternatives/1 is located in ExampleWeb.Router.RoutexHelpers which is aliased as Routes -->\n <.link\n   :for={alternative <- Routes.alternatives(@url)}\n   class=\"button\"\n   rel=\"alternate\"\n   hreflang={alternative.attrs.locale}\n   patch={alternative.slug}\n >\n   <.button class={(alternative.match? && \"bg-[#FD4F00]\") || \"\"}>\n     <%= alternative.attrs.display_name %>\n   </.button>\n </.link>\n```","ref":"localized_routes.html#step-4-show-alternative-pages-to-the-user"},{"type":"extras","title":"Conclusion - Localized Routes","doc":"In this tutorial you have learned how to create localized routes for your\nPhoenix application using multiple extensions and how to add custom attributes\n(such as `:locale` and `:display_name`) to these routes. There are a few more\nextension you can add to the mix for extra flexibility and convenience, such as:\n\n* **Routex.Extension.Interpolation** - Use any attribute to customize routes\n  (e.g. \"/#{locale}/products/#{display_name}/:id/edit\")\n* **Routex.Extension.Assigns** - Use any attribute in your templates using `@`\n  notation.\n* **Routex.Extension.AttrGetters** - Lazy load attributes\n\nIf you encounter any issues with Routex or this tutorial, feel free to [open a topic at Elixir\nForums](https://elixirforum.com/tag/routex) or create an issue at GitHub.\n\nHave a nice day!","ref":"localized_routes.html#conclusion"},{"type":"extras","title":"Localization vs Translation","doc":"# **Localization vs. Translation: Why Your Website Should Keep Them Separate**\n\nWhen expanding a website for a global audience, businesses often confuse\n**translation** with **localization**. While they are related, treating them as\nthe same process can lead to usability issues and a poor user experience.\nAdditionally, many websites make the mistake of assuming that a user's\n**preferred language** matches their **physical location**, which can cause\nfrustration.\n\nIn this post, we’ll break down the differences between translation and\nlocalization, why your website should separate them, and why **language\npreferences should not be tied to a user's location**.","ref":"localization_vs_translation.html"},{"type":"extras","title":"The Problem: Conflating Location and Language - Localization vs Translation","doc":"Many websites make the mistake of assuming that location dictates language.\nWhile there's often a correlation, it's far from a perfect match. Think about\nit:\n\n* **Multilingual Regions:** Countries like Switzerland, Canada, and Belgium have\n  multiple official languages. A user in Switzerland might prefer to browse in\n  German, French, or Italian. Assuming their language based on their IP address\n  (which indicates location) would be inaccurate.\n\n* **Expatriates and Travelers:** Someone living abroad might prefer to browse in\n  their native language, even if they're physically located in a different\n  country. A German expat in Spain might still want to see the website in\n  German.\n\n* **Language Learning:** Some users might prefer to browse in a language they're\n  learning, regardless of their location.\n\n* **Shared Computers:** In internet cafes, libraries, or shared family\n  computers, users might not have control over the browser's language settings.\n  Relying on these settings can lead to an incorrect language selection.\n\n**The Problem with `Accept-Language` HTTP Header:**\n\nWebsites often use the `Accept-Language` HTTP header, sent by the browser, to\ndetermine the user's preferred language. While this can be helpful, it's not\nfoolproof. As mentioned above, on shared computers, the `Accept-Language` header\nmight reflect the preferences of a previous user. Users might also not know how\nto change this setting, or it might be locked by system administrators in\ncertain environments. Therefore, relying solely on this header can lead to a\nfrustrating experience.\n\n**Examples of What *Not* to Do:**\n\n* **Automatic Redirection Based on IP:** A user in Canada is automatically\n  redirected to the French version of the site, even though their browser and\n  system language are set to English. This is a classic example of location\n  overriding language preference.\n\n* **Flag Icons as Language Options:** Using flag icons to represent language is\n  problematic. Flags represent countries, not languages. What about Spanish\n  speakers in the US? Or English speakers in India? This conflates nationality\n  with language.\n\n* **Hidden Language Settings:** Language options are buried deep in the footer\n  or only appear after navigating through several pages. Users shouldn't have to\n  hunt for their preferred language.\n\n* **Sole Reliance on `Accept-Language`:** The website assumes the browser's\n  language setting is the user's actual preference, ignoring the possibility of\n  shared computers or incorrect settings.","ref":"localization_vs_translation.html#the-problem-conflating-location-and-language"},{"type":"extras","title":"The Solution: Always Give Users Control - Localization vs Translation","doc":"The key is to treat location and language as distinct, yet related, pieces of\ninformation *and always give users explicit control over both*. Here's how to do\nit right:\n\n* **Explicit Language Selection:** Provide clear and prominent language options,\n  ideally using the language name itself (e.g., \"English,\" \"Español,\" \"Deutsch\")\n  rather than flags. Place these options in a visible location, such as the\n  header or footer, *on every page*.\n\n* **Location as a Secondary Consideration:** Use location data (IP address) to\n  *suggest* a default language and/or currency, but *always* allow the user to\n  override this suggestion. A simple popup or banner saying \"We've detected\n  you're in [Location]. Would you like to view the site in [Suggested Language]?\n  [Yes/No]\" is a good approach. Even if they click \"yes,\" the language option\n  should *still* be readily available.\n\n* **User Profiles and Preferences:** For returning users, store their language\n  and location preferences in their user profile. This ensures a consistent\n  experience across sessions.\n\n* **Content Localization, Not Just Translation:** Consider cultural nuances and\n  adapt content accordingly. Simply translating text without considering\n  cultural context can be ineffective or even offensive. Dates, times, and units\n  of measurement should also be localized.\n\n* **Clear Location Settings:** If location-specific content is crucial (e.g.,\n  store locator, shipping information), provide a separate and easy-to-use\n  location selection mechanism. This could be a dropdown menu or a map\n  interface.\n\n**Example of How to Do It Right:**\n\n* A user lands on a website and sees a small popup: \"We've detected you're in\n  the UK. Would you like to view prices in GBP and the site in English?\n  [Yes/No]\"\n\n* *Regardless* of the user's choice in the popup, a language dropdown menu is\n  *always* visible in the header, offering options like \"English,\" \"Français,\"\n  \"Español,\" etc.\n\n* The footer contains a link to \"Change Location,\" where the user can specify\n  their country for location-specific content.\n\nBy implementing these best practices, websites can create a more inclusive and\nuser-friendly experience for their global audience. Respecting the distinction\nbetween location and language, and *always giving users the control to choose*,\nis not just good practice, it's essential for building trust and maximizing your\nonline reach.","ref":"localization_vs_translation.html#the-solution-always-give-users-control"},{"type":"extras","title":"How Routex' approach helps - Localization vs Translation","doc":"Routex's approach to localized routing reinforces the principle of keeping\nlanguage and location distinct. No implicit information is embedded in the\nroutes.\n\nWhen dealing with **region-based pages** changing a user's region (and thus the\nassociated region-specific content) doesn't necessitate an automatic language\nswitch. And when dealing with **language-based pages** changing a user's\nlanguage doesn't necessitate an automatic region switch.\n\nImagine a scenario where a user is browsing a **region-based** site in English\nbut wants to see the pricing and product availability for the Indian market.\nWith Routex, they can navigate to the India region-specific page (e.g.,\n/in/products) without being forced to switch to another language. The site can\nmaintain the user's preferred language (English in this case) while displaying\nthe relevant Indian content.\n\nThis is in stark contrast to systems where language and region are implicitly\nlinked. In such cases, switching regions might inadvertently trigger a language\nchange, leading to a confusing and frustrating user experience.\n\n\n## **Conclusion**\n\nLocalization and translation serve different purposes and should be handled\nseparately to provide the best user experience. Similarly, **a user's preferred\nlanguage should not be assumed based on their location**. By keeping these\nelements distinct, websites can ensure better usability, compliance, and\nengagement for a global audience.\n\nBy adopting **a user-first approach**, where language is a choice and location\nis used only for relevant regional settings, businesses can create a seamless,\naccessible, and culturally appropriate experience for all users.","ref":"localization_vs_translation.html#how-routex-approach-helps"},{"type":"extras","title":"Routex and Phoenix Router","doc":"# How Routex and Phoenix Router Work Together: A Blueprint for Understanding\n\n \n All code on this page is for illustration purpose only. It is incorrect and\n should not be used.\n \n\nHow Routex, its extensions, and the Phoenix Router work together can be better\nunderstood through an analogy. As the saying goes, \"a picture is worth a\nthousand words,\" this document also includes an illustrative blueprint.","ref":"routex_and_phoenix_router.html"},{"type":"extras","title":"Analogy: the housing project - Routex and Phoenix Router","doc":"Imagine you're a *real estate developer* planning to build several houses. You\nhave a general vision for the houses (your route definitions in `route.ex`) and\nsome specific ideas about their style and features (your Routex configuration).\n\n```\n# routes\n/products\n/products/:id\n\n# config\nalternatives: %{\n    \"/fr\" => %{name: \"French\"},\n    \"/es\" => %{name: \"Spanish\"}\n}\n```\n\nRoutex is the *architect*. It takes your vision (route definitions) and\npreferences (Routex config) and creates detailed blueprints.\n\nRoutex extensions are the *architect's specialized tools*. These tools allow the\narchitect to refine and customize the blueprints. Without them, the architect\ncould only create basic, unmodified plans.\n\n```\n# input\n[\n%Route{path: \"/products\"}\n%Route{path: \"/products/:id\"}\n]\n\n# output after transformation by Alternatives extension\n[\n%Route{path: \"/fr/products\",      metadata: %{name: \"French\"}},\n%Route{path: \"/fr/products/:id\",  metadata: %{name: \"French\"}},\n%Route{path: \"/es/products\",      metadata: %{name: \"Spanish\"}},\n%Route{path: \"/es/products/:id\",  metadata: %{name: \"Spanish\"}}\n]\n```\n\nOnce the blueprints are finalized, they're handed off to the *construction\ncompany*: Phoenix Router. Phoenix Router builds the actual houses (your routes)\naccording to the architect's precise specifications. The blueprints are\nperfectly formatted for Phoenix Router, ensuring a smooth construction process.\n\n```\n# note: incorrect pseudo code\n\nif match?(\"/fr/products\"      ), do: ProductLive, :index, metadata: [\"French\"]\nif match?(\"/fr/products/\" <> id), do: ProductLive, :show,  metadata: [\"French\"]\nif match?(\"/es/products\"      ), do: ProductLive, :index, metadata: [\"Spanish\"]\nif match?(\"/es/products/\" <> id), do: ProductLive, :show,  metadata: [\"Spanish\"]\n```\n\nThis explains the first key concept:\n\n> **Routex generates blueprints from your route definitions and configuration,\n> ready for Phoenix Router to build the actual routes.**\n\nBecause Routex is the architect, it has intimate knowledge of the house designs.\nThis allows it to create perfectly matching accessories, like custom-designed\nsunshades or smart garage doors. These are *additional* features that enhance\nthe houses built by Phoenix Router, adding convenience and functionality.\n\n```\n# generated convenient functions\ndefmodule Helpers do\n  def alternatives(\"/products\") do\n    [\n      %Route{path: \"/fr/products\", name: \"French\"},\n      %Route{path: \"/es/products\", name: \"Spanish\"}\n    ]\n  end\nend\n\n# your usage\nThis page is available in:\nfor alt <- Helpers.alternatives(\"/products\") do\n <.link navigate={alt.path}>{alt.name}<./link>\nend\n\n# output\nThis page is available in:\n French \n Spanish \n```\n\nThis leads to the second key concept:\n\n> **Routex also creates helpful accessory functions that you can use with the\n> houses (routes) built by Phoenix Router.** These functions streamline common\n> tasks and improve the overall experience.","ref":"routex_and_phoenix_router.html#analogy-the-housing-project"},{"type":"extras","title":"Example blueprint - Routex and Phoenix Router","doc":"A picture paints a thousand words, or so they say. The blueprint clearly shows\nhow Routex is middleware, plugged between two stages of Phoenix route generation.\n\nAlso shown is the use two co-operating extensions: `Translations` uses the\n`:language` attribute set by `Alternatives`.\n\n \nflowchart TD\n subgraph subGraph1[\"Routex\"]\n        F[\"ExampleWeb.RoutexBackend.ex\"]\n        G[\"configure/2 callbacks\"]\n        H[\"Alternatives.transform/3 callback\"]\n        I[\"Translations.transform/3 callback\"]\n        J[\"create_helpers/3 callbacks\"]\n     K[\"ExampleWeb.Router.RoutexHelpers\"]\n  end\n  subgraph subGraph0[\"Phoenix\"]\n        A[\"ExampleWeb.Router.ex\"]\n        B[\"Convert to Phoenix.Routes.Route structs\"]\n        C[\"Generate route functions\"]\n        D[\"ExampleWeb.Router\"]\n  end\n  A -- \"/products/:id\" --> B\n  B -- \"%{path: /products/:id}\" --> H\n  F -- \"extensions: [Alternatives, Translations]\" --> G\n  G --> H\n  H -- \"%{path: ..., attrs: %{lang: fr}}\n       %{path: ..., attrs: %{lang: es}}\" --> I\n  I -- \"%{path: /produit/:id, attrs: ...}\n       %{path: /producto/:id, attrs: ...}\" --> C & J\n  C -- \"def match?(/produit/:id), do: ProductController\n      def match?(/producto/:id), do: ProductController\" --> D\n  J -- \"def alternatives(/products/:id) do\n        %{path: /produit/:id, attrs: %{lang: fr}}\n        %{path: /producto/:id, attrs: %{lang: es}}\n      end\" --> K\n\n  F:::routex\n  G:::routex\n  H:::routex\n  I:::routex\n  J:::routex\n  K:::routex\n  A:::phoenix\n  B:::phoenix\n  C:::phoenix\n  D:::phoenix\n classDef routex  fill:#4d4d4d, stroke:#888, stroke-width:2px, color:#ffffff\n classDef phoenix  stroke-width:1px, stroke-dasharray: 0, stroke:#616161, fill:#FF6D00, color:#424242\n style subGraph0 fill:#FFFFFF, stroke:#757575\n style subGraph1 fill:#FFFFFF, stroke:#757575","ref":"routex_and_phoenix_router.html#example-blueprint"},{"type":"extras","title":"Conclusion - Routex and Phoenix Router","doc":"Phoenix Router and Routex, along with its extensions, form a powerful\npartnership that empowers you to build any route in your web application. Routex\nexpertly designs the intricate pathways of your application's URLs, while\nPhoenix Router flawlessly constructs them, ensuring a smooth and efficient user\nexperience.","ref":"routex_and_phoenix_router.html#conclusion"},{"type":"extras","title":"Routing solutions compared","doc":"# Phoenix Framework routing solutions compared\n\nThis guide is intended to help you understand the differences, strengths,\nand tradeoffs when deciding which routing solution best fits your needs.","ref":"comparison.html"},{"type":"extras","title":"Phoenix Router - Routing solutions compared","doc":"Phoenix Router is the built‐in router provided by the Phoenix framework. Since\nPhoenix 1.8 it offers simple path prefixing (e.g. using :path_prefixes for\nlanguage segments), but it does not support full URL translation.\n\n**Stick to Phoenix Router if…**  \nYour application has straightforward routing needs and you prefer to stick with\nthe built-in, battle-tested routing system without extra dependencies.","ref":"comparison.html#phoenix-router"},{"type":"extras","title":"Cldr Routes - Routing solutions compared","doc":"Cldr Routes extends the Phoenix Router by adding support for URL translation.\nIts tight coupling with the Cldr ecosystem ensures compatibility but that comes\nat the cost of introducing a heavy dependency. While Cldr Routes handles\ntranslated routes well, its feature set is focused mainly on\ninternationalization and does not offer much beyond that scope.\n\n**Choose Cldr Routes if…**  \nYour sole need is to support multilingual URLs and you are already invested in\nthe Cldr ecosystem.","ref":"comparison.html#cldr-routes"},{"type":"extras","title":"Routex - Routing solutions compared","doc":"Routex takes routing in Phoenix to the next level. It is built as\n[extension-driven](EXTENSION_SUMMARIES.md) middleware leveraging Phoenix Router\nitself. Its extension-driven design means you get all the benefits of translated\nroutes—like those offered by Cldr Routes—plus a broader, more customizable set\nof features that can be tailored to virtually any routing challenge.\n\nWhether you need complete route translation, advanced route manipulation,\nobfuscation, or alternative route generation, Routex provides a full suite of\ntools without adding unnecessary runtime overhead or dependencies.\n\n**Choose Routex if…**  \nRoutex is best if you foresee the need for extensive route transformations\n(including translation), dynamic alternative routes, or if you want to avoid the\nheavier dependencies imposed by Cldr Routes.\n\nps. If you invested in the Cldr ecosystem but need to use features only offered by Routex,\nbridge the gap using [the Cldr extension for Routex](`Routex.Extension.Cldr`).","ref":"comparison.html#routex"},{"type":"extras","title":"Comparison table - Routing solutions compared","doc":"| Feature             | Routex     | Cldr Routes         | Phoenix >= 1.8 |\n|---------------------|------------|---------------------|----------------|\n| Route encapsulation | Full  [^1] | Limited             | Limited        |\n| Route manipulation  | Full  [^2] | Limited             | Limited        |\n| Route interpolation | Full       | Limited             | Prefix only    |\n| Alternative Routes  | Full       | Translation focused | Prefix only    |\n| Verified Routes     | ☑          | ☑                   | ☑              |\n| Translation         | ☑          | ☑                   | ☐              |\n| Route Helpers       | ☑          | depr.               | depr.          |\n| Drop-in replacement | ☑     [^3] | ☐                   | -              |\n| Standalone          | ☑          | ☐                   | -              |\n| Modular             | ☑          | ☐                   | -              |\n| Extendable          | ☑          | ☐                   | -              |\n\n[^1]: Routex' `preprocesss_using` is not bound to Phoenix (session) scopes  \n[^2]: [Crazy example](https://hexdocs.pm/routex/Routex.Extension.Cloak.html)  \n[^3]: *Optionally* Routex can be configured to shim original Phoenix functionality (for example: `~p` and `url/2`) or\nmimick Cldr Routes using [an adapter extension](https://hexdocs.pm/routex/Routex.Extension.Cldr.html).","ref":"comparison.html#comparison-table"},{"type":"extras","title":"Troubleshooting","doc":"# Troubleshooting","ref":"troubleshooting.html"},{"type":"extras","title":"Where can I find ExampleWeb.Router.RoutexHelpers? - Troubleshooting","doc":"This module does not have a code file. It is generated during compile time\nby Routex. You should be able to see a message in the output like the one below:\n\n    Completed: ExampleWeb.RoutexCldrBackend ⇒ Routex.Extension.VerifiedRoutes.create_helpers/3\n    Create or update helper module ExampleWeb.Router.RoutexHelpers\n\nOnce your project is compiled, you can access it in `iex`.\n\n    iex> exports ExampleWeb.Router.RoutexHelper\n    alternatives/1                     attrs/1                            on_mount/4\n    sigil_o/2                          sigil_p/2                          url/1\n    url/2                              url/3                              url_phx/1\n\n    iex> h ExampleWeb.Router.RoutexHelper.attrs\n\n              def attrs(url)\n    Returns Routex attributes of given URL","ref":"troubleshooting.html#where-can-i-find-exampleweb-router-routexhelpers"},{"type":"extras","title":"Compilation - Troubleshooting","doc":"If you run into compilation issues try these solutions first. If they\ndo not solve the issue or the issue re-appears, fell free to open an issue.","ref":"troubleshooting.html#compilation"},{"type":"extras","title":"Clearing your _build folder - Troubleshooting","doc":"Clearing your build folder might fix issues; especially when the\norder of module compilation is the suspect.\n\n`rm -Rf _build && mix compile`","ref":"troubleshooting.html#clearing-your-_build-folder"},{"type":"extras","title":"Debugging - Troubleshooting","doc":"When your application fails to compile you might find the cause by setting the\nenvironment variable `ROUTEX_DEBUG` to `true`.\n\n    ROUTEX_DEBUG=true mix compile\n\nDo note that this might show early compilation issues, but will make the final\ncompilation fail at all times.","ref":"troubleshooting.html#debugging"},{"type":"extras","title":"Changelog","doc":"# Change Log\n\nAll notable changes to this project will be documented in this file.\nSee [Conventional Commits](Https://conventionalcommits.org) for commit guidelines.\n\n<!-- changelog -->\n\n## [1.0.0](https://github.com/BartOtten/routex/compare/v1.0.0...1.0.0) (2025-02-03)","ref":"changelog.html"},{"type":"extras","title":"Features: - Changelog","doc":"* support Phoenix Liveview >= 1.0","ref":"changelog.html#features"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* ci: upgrade artifact actions in workflow\n\n* core: comp. error - cannot set :__struct__ in struct definition\n\n* incorrect typespecs\n\n* cldr: use territory_from_locale for territory resolution\n\n## [v0.3.0-alpha.4](https://github.com/BartOtten/routex/compare/v0.3.0-alpha.3...v0.3.0-alpha.4) (2024-11-04)\n\n\n\n\n## [v0.3.0-alpha.3](https://github.com/BartOtten/routex/compare/v0.3.0-alpha.2...v0.3.0-alpha.3) (2024-10-28)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Features: - Changelog","doc":"* cldr: add adaptor for projects using Cldr","ref":"changelog.html#features"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* core: preprocess_using does not accept options\n\n* mix: do not override dep :ssl_verify_fun\n\n## [v0.3.0-alpha.2](https://github.com/BartOtten/routex/compare/v0.3.0-alpha.1...v0.3.0-alpha.2) (2024-10-23)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Breaking Changes: - Changelog","doc":"* dev: rename development aid functions\n\n* alternatives: rename :is_current? to :match?\n\n* translations: depend on Gettext greater than 0.26.0","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* docs: warnings about referencing Phoenix.Router.Route\n\n* core: rename is_private/1 to private?/1\n\n* ci: update Github actions\n\n* core: warning during compilation about usage of :warn\n\n## [v0.3.0-alpha.1](https://github.com/BartOtten/routex/compare/v0.3.0-alpha.1...v0.3.0-alpha.1) (2024-10-21)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Breaking Changes: - Changelog","doc":"* core: remove Routex.Path module\n\n* core: split Extension Utils module\n\n* alternatives: rename scope to branch","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Features: - Changelog","doc":"* core: merge private Routex attrs into socket\n\n* verified: branching macro's of all arities of ~p, url and path\n\n* core: enable AST-free manipulation of routes with interpolation\n\n* alternatives: indicate if an alternative route `is_current?`\n\n* interpol: add extension for interpolation of routes\n\n* translations: distinct locale and language\n\n* core: introduce Branching module\n\n* core: introduce Matchable module\n\n## [v0.2.0-alpha.8](https://github.com/BartOtten/routex/compare/v0.2.0-alpha.7...v0.2.0-alpha.8) (2023-10-18)","ref":"changelog.html#features"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* translations: reliably extract segments to translate\n\n## [v0.2.0-alpha.7](https://github.com/BartOtten/routex/compare/v0.1.0-alpha.7...v0.2.0-alpha.7) (2023-05-31)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Features: - Changelog","doc":"* utils: use verbose defaults for esc_inspect/2","ref":"changelog.html#features"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* route: swallowed routes due to nonunique map key\n\n## [v0.1.0-alpha.6](https://github.com/BartOtten/routex/compare/v0.1.0-alpha.5...v0.1.0-alpha.6) (2023-05-30)\n\n\n\n\n## [v0.1.0-alpha.5](https://github.com/BartOtten/routex/compare/v0.1.0-alpha.5...v0.1.0-alpha.5) (2023-05-05)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Features: - Changelog","doc":"* add ROUTEX_DEBUG for compilation debugging (#3)","ref":"changelog.html#features"},{"type":"extras","title":"Bug Fixes: - Changelog","doc":"* verified: let Phoenix handle missing routes","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Extensions","doc":"# Routex Extensions\n\n> #### List of extensions {: .info}\n> A [list of included extensions](README.md#extensions) can be found in the README.\n\nRoutex Extensions extend the functionality provided by Routex to transform\nroutes or generate new route based helper functions. Each extension is a module\nwhich implements the `Routex.Extension` behaviour. It has to implement one or\nmultiple callbacks:\n\n- configure/2\n- transform/3\n- post_transform/3\n- create_helpers/3\n\nRoutex will call those callbacks at different stages before Routex handsoff the\nresult to `Phoenix.Router` for compilation.","ref":"extension_development.html"},{"type":"extras","title":"Callbacks and stages - Extensions","doc":"","ref":"extension_development.html#callbacks-and-stages"},{"type":"extras","title":"Stage 1: Configure - Extensions","doc":"This stage enables extensions to preprocess backend options upfront.\n\nThe `configure/2` callback is called with the options provided to\n`Routex.Backend` and the name of the Routex backend. It is expected to return a\nnew list of options.\n\nRoutex collects all options in this stage for subsequent stages. Therefore,\nextensions should add any fallback/default they might use themselves to the\noptions in this stage.\n\nTo aid in code completion, the final configuration is passed as a struct to\nsubsequent stages.","ref":"extension_development.html#stage-1-configure"},{"type":"extras","title":"Stage 2: Transform - Extensions","doc":"This stage is meant to change the properties of routes, which are at that moment\n`Phoenix.Router.Route` structs. The routes are grouped by Routex backend and\nprocessed per group, allowing an extension to use accumulating values within one\niteration.\n\nThe `transform/3` callback is called with a list of routes belonging to a\nRoutex backend, the name of the backend and the current environment. It is\nexpected to return a list of Phoenix.Router.Route structs.\n\n#### Flattening option values\nExtensions can make use of `Routex.Attrs` provided by Routex itself, Routex\nbackends and other extensions.\n\nTo make the availability of the attributes as predictable as possible, Routex\nuses a flat structure which is stored in a routes' `private.routex` field.\nHowever, using a flat structure might conflict with developer experience;\nsometimes a nested structure to provide configuration options might be more\nsuitable.\n\nOne responsibility of the `transform/3` callback is to flatten the structure of\nattributes they use for each route they receive, so other extensions can use\nattributes set by the current extension without knowledge of the configuration\nstructure.\n\n**Example**\nThe Alternatives extension uses nested options and allows inheritance\nof attributes from parent branches.\n\n```\nalternatives: %{\n  \"/\" =>\n    helper: nil,\n    locale: \"en_GB\",\n    branches: %{\n      \"nl\" => %{\n          helper: \"nl\",\n          locale: \"nl_NL\"\n        },\n      \"gb\" => %{\n        helper: \"gb\",\n        }\n    }\n}\n```\nThe Alternatives module is therefor responsible for flattening those for\n(itself and) other extensions to use. To take the route responsible for the\n\"gb\" branch as an example, the extension should add flattened attributes in the\nRoute struct. It can do so using the helper function `Routex.Attrs.put/2`.\n\n```\nRoutex.Attrs.put(route, [locale: \"en_GB\", helper: \"gb\"])\n```\n\nNow the `Translation` extension can search for the option `:locale` in the\nroute's attributes, unaware of how that locale was initially configured.","ref":"extension_development.html#stage-2-transform"},{"type":"extras","title":"Stage 3: Post Transform - Extensions","doc":"The `post_transform` stage is meant to set `Routex.Attrs` knowing all other\nproperties of a route are available.","ref":"extension_development.html#stage-3-post-transform"},{"type":"extras","title":"Stage 4: Create helpers - Extensions","doc":"In this stage helper functions can be generated which will be added to\n`MyAppWeb.Router.RoutexHelpers`.\n\nThe `create_helpers/3` callback is called with a list of routes belonging to a\nRoutex backend, the name of the Routex backend and the current environment.\nIt is expected to return Elixir AST.\n\nAs a result the developer only has to `import MyAppWeb.Router.RoutexHelpers`\nfor all helpers generated by extensions to be included in the app.","ref":"extension_development.html#stage-4-create-helpers"},{"type":"extras","title":"Guidelines - Extensions","doc":"* make functions not defined by the `Routex.Extension` behaviour private.\n* provide as many options as possible; other extensions might use the information.\n* provide additional options as flat list(s) so other extensions don't have to guess structure.\n* as other extensions might use options set by your extension, try to preserve any previously defined option.","ref":"extension_development.html#guidelines"},{"type":"extras","title":"Documentation - Extensions","doc":"@moduledoc \"\"\"\n    Summary of feature provided.","ref":"extension_development.html#documentation"},{"type":"extras","title":"Options - Extensions","doc":"- `name` - description","ref":"extension_development.html#options"},{"type":"extras","title":"Example configuration - Extensions","doc":"```diff\n    # file /lib/example_web/routex_backend.ex\n    defmodule ExampleWeb.RoutexBackend do\n      use Routex.Backend,\n      extensions: [\n    +   Routex.Extension.Name\n        Routex.Extension.Attrs\n    ],\n    + name_config: [name_opt: \"value\"]\n    ```","ref":"extension_development.html#example-configuration"},{"type":"extras","title":"Pseudo result - Extensions","doc":"/products/:id/edit  ⇒ /products/:id/edit\n\n    ## `Routex.Attrs`\n    **Requires**\n    - none\n\n    **Sets**\n    - none","ref":"extension_development.html#pseudo-result"},{"type":"extras","title":"Helpers - Extensions","doc":"function_name(arg1 :: type) :: type\n    \"\"\"","ref":"extension_development.html#helpers"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}